#!/usr/bin/env python
# KMB 11/06/2008


import os # use os for manipulating path names
import sys # use sys for parsing command line
import time # use time for setting playback rate

# we need to import pyglet.media before scipy.linalg.decomp is 
# imported by kcluster, as this seems to cause have_avbin to be false
# on windows
# and now we need to import ctypes before media because this seems to 
# be necessary to be able to find the avbin dll on windows. 
# man!
try:
    if sys.platform.startswith( 'win' ):
        import ctypes
        avbin = ctypes.cdll.avbin
except:
    pass

# KB: use commandline to set whether AVBIN is used
import version

import wx
from wx import xrc
import numpy as num
num.seterr( invalid='raise' ) # raise errors on numerical exceptions
num.seterr( divide='raise' )
num.seterr( over='warn' )
num.seterr( under='ignore' )
from scipy.misc import imresize

from version import __version__, DEBUG, DEBUG_REPEATABLE_BEHAVIOR
import annfiles as annot
import batch
import bg
import algorithm
import draw
import imagesk
import movies
import ellipsesk as ell
from params import params, const

if DEBUG:
    import pdb

class CtraxApp (algorithm.CtraxAlgorithm): # eventually inherits from wx.App
    def has( self, attr ):
        return hasattr( self, attr ) and getattr( self, attr ) is not None
    
    def OnInit( self ):
	"""
	Start up the Ctrax GUI
	"""
        self.InitState() # in settings.py

        self.ParseCommandLine()

        self.InitGUI() # in settings.py

        # read saved settings, overwriting defaults
        self.ReadUserfile() # in settings.py

        # draw GUI
        self.frame.Show()
        self.alive = True
        params.app_instance = self

        if params.interactive:
            print "********** Ctrax Warning and Error Messages **********"
            print "Error and warning messages will appear in this window."
            print "If you have trouble and are contacting the Ctrax mailing"
            print "list (see http://groups.google.com/group/ctrax ), be"
            print "sure to copy and paste the relevant messages from this"
            print "window into your email. Your Ctrax version is %s."%__version__
            print "*********************** Ctrax ************************\n"
            print "using wx version", wx.__version__
            from pyglet import version as pyglet_version
            print "using pyglet version", pyglet_version

        # open movie, ann file
        self.OpenMovie( new_movie=False )

        return True


    def PrintUsage(self):
	"""
	Print command line arguments for Ctrax.
	"""
        self.RestoreStdio()
        print """Ctrax:
Optional Command Line Arguments:
--Interactive={True,False}
--Output=<movie.fmf.ann>
--Input=<movie.fmf>
--SettingsFile=<settings.ann>
--AutoEstimateBackground={True,False}
--AutoEstimateShape={True,False}
--AutoDetectCircularArena={True,False}
--CompressMovie=<movie.sbfmf>
--Matfile=<movie.mat>
--DiagnosticsFile=<movie_ctraxdiagnostics.txt>
--FirstFrameTrack={0,1,...}
--LastFrameTrack={-1,0,1,...}
--ResumeTracking={True,False}
--UseAVBIN={Always,Allow,Never}

Example:
Ctrax --Interactive=True --Input=movie1.fmf \\
  --Output=movie1.fmf.ann \\
  --SettingsFile=exp1.ann \\
  --Matfile=movie1.mat \\
  --DiagnosticsFile=movie1_ctraxdiagnostics.txt

By default, Interactive=True, AutoEstimateBackground=True,
AutoEstimateShape=True, AutoDetectCircularArena=True, 
FirstFrameTrack=0, LastFrameTrack=-1 
(meaning to track until the end of the video),
ResumeTracking=False, UseAVBIN=Allow

If not in interactive mode, then Input must be defined.

If Input is movie1.fmf and Output is not defined, then output
and settings will go to movie1.fmf.ann in non-interactive mode.

Existing annotation files are always backed up before being
overwritten, as long as the user has the appropriate
permissions to the output directory.

If CompressMovie is not set, then a compressed SBFMF will not
be created.

If Matfile is not set, then <basename>.mat will be used,
where <basename> is the base name of the movie.

If DiagnosticsFile is not set, then
<basename>_ctraxdiagnostics.txt will be used.

If UseAVBIN is Always, then Ctrax will not use its own uncompressed 
AVI reader, and will instead use AVBIN's. 
If UseAVBIN is Never, then Ctrax will not use AVBIN and will not be
able to read compressed AVIs. 
If UseAVBIN is Allow, then Ctrax will read uncompressed AVIs with
its own reader, and compressed AVIs with AVBIN. This is the normal
behavior of Ctrax when in interactive mode.

"""

    def ParseCommandLine(self):
	"""
	Interpret command line arguments.
	"""
        args = sys.argv[1:]

        self.movie = None
        self.ann_filename = None
        self.start_frame = 0
        # KB 20120109: added support for last_frame command-line argument
        self.last_frame = num.inf
        self.dowritesbfmf = False

        self.input_file_was_specified = False
        self.output_file_was_specified = False

        if len( args ) == 1:
            if args[0] == '--help':
                self.PrintUsage()
                exit( 1 )
            elif args[0].startswith( '-psn_' ):
                # in Mac, launching from Finder sends a process serial number
                args = []
            elif not '=' in args[0]:
                # in Windows, it could be a bare movie name
                args = ['--input=%s'%args[0]]

        # parse first for arguments that could alter other arguments' effects
        for arg in args:
            if arg.lower() == '--interactive=false':
                params.interactive = False
                params.enable_feedback( False )
                # if we were redirecting to an output window,
                # restore stdio to the command line prompt
                self.RestoreStdio()
            
        for i in range(len(args)):
	    # the arguments should be of the form --<paramname>=<paramvalue>
            try:
                name,value = args[i].split('=',1)
            except:
                print 'Error parsing command line arguments. No equals sign found. Usage: '
                self.PrintUsage()
                raise NotImplementedError
            if name.lower() == '--interactive':
                continue # already handled
            elif name.lower() == '--input':
                if hasattr( self, 'frame' ):
                    openframe = self.frame
                else:
                    openframe = None
                self.movie = movies.Movie( value, interactive=params.interactive,
                                           parentframe=openframe, open_now=True )
                self.input_file_was_specified = True
            elif name.lower() == '--output':
                self.ann_filename = value
                self.output_file_was_specified = True
            elif name.lower() == '--settingsfile':
                self.settingsfilename = value
                print "settingsfile = " + str(self.settingsfilename)
            elif name.lower() == '--autoestimatebackground':
                if value.lower() == 'false':
                    params.batch_autodetect_bg_model = False
            elif name.lower() == '--autoestimateshape':
                if value.lower() == 'false':
                    params.batch_autodetect_shape = False
            elif name.lower() == '--autodetectcirculararena':
                if value.lower() == 'false':
                    params.batch_autodetect_arena = False
                elif value.lower() == 'none':
                    params.batch_autodetect_arena = None
            elif name.lower() == '--compressmovie':
                self.writesbfmf_filename = value
                self.dowritesbfmf = True
            elif name.lower() == '--matfile':
                self.matfilename = value
            elif name.lower() == '--diagnosticsfile':
                self.diagnostics_filename = value
            elif name.lower() == '--firstframetrack':
                # KB 20120109: added support for start_frame command-line argument
                try:
                    start_frame = float(value)
                    if start_frame < 0 or round(start_frame) != start_frame:
                        raise NotImplementedError
                except:
                    print "FirstFrameTrack must be an integer greater than or equal to 0"
                    self.PrintUsage()
                    raise
                self.start_frame = int(start_frame)
                print "setting params.start_frame = " + str(int(start_frame))
                params.start_frame = int(start_frame)
            elif name.lower() == '--lastframetrack':
                try:
                    last_frame = float(value)
                    if round(last_frame) != last_frame:
                        raise NotImplementedError
                    if last_frame < 0:
                        last_frame = num.inf
                except:
                    print "LastFrameTrack must be an integer"
                    self.PrintUsage()
                    raise
                self.last_frame = last_frame
            elif name.lower() == '--resumetracking':
                if value.lower() == 'true':
                    params.noninteractive_resume_tracking = True
            elif name.lower() == '--useavbin':
                # KB: add UseAVBIN parameter, only import AVBIN if 
                if value.lower() == 'never':
                    version.USE_AVBIN = False
                else:
                    version.USE_AVBIN = True
                    if value.lower() == 'always':
                        params.use_uncompressed_avi = False
                    import pyglet.media as media

            else:
                print 'Error parsing command line arguments. Unknown parameter name "%s". Usage: '%name
                self.PrintUsage()
                raise NotImplementedError

        if self.start_frame > self.last_frame:
            print "FirstFrameTrack must be <= LastFrameTrack"
            self.PrintUsage()
            raise NotImplementedError

        if params.noninteractive_resume_tracking:
            # to resume tracking, output file must exist
            # (and should have ann data in it)
            if (not self.output_file_was_specified) or \
                   self.ann_filename is None or \
                   (not os.path.isfile( self.ann_filename )):
                print "To resume tracking, an existing output file must be specified."
                raise NotImplementedError

        # run noninteractive mode
        if params.interactive == False:
            self.run_noninteractive()
            exit( 0 )


    def run_noninteractive(self):
	"""
	Run Ctrax in non-interactive mode.
	"""
        starttime = time.time()
        
        self.frame = None

        # input movie name must be specified on the command line
        if not self.has( 'movie' ):
            print 'Error parsing command line arguments.\n\
            Input file must be specified in non-interactive mode.\n\
            Usage: '
            self.PrintUsage()
            raise NotImplementedError

        if not self.has( 'ann_filename' ):
            self.ann_filename = self.get_filename_with_extension( '+.ann' )

        if self.has( 'ann_filename' ):
            print "ann_filename = " + str(self.ann_filename)

        if not self.has( 'settingsfilename' ):
	    self.settingsfilename = self.ann_filename

        # open the movie
        print "Opening movie " + self.movie.filename
        self.OpenMovie( new_movie=False )

        # start_frame can be reset by annfile if output file exists?
        params.start_frame = self.start_frame

        # KB 20120109: do_refresh not set otherwise
        self.do_refresh = True
        # actually, it's previously set to False in InitState() -JAB 1/10/12
        # does it need to be True? what happens if it's not set here?
        
        if not self.has('diagnostics_filename'):
            self.diagnostics_filename = self.get_filename_with_extension( '_ctraxdiagnostics.txt' )
        print "Diagnostics info will be written to " + self.diagnostics_filename

	# do the tracking steps
        print "DoAll..."
        self.DoAll()

        print "process completed in", time.time()-starttime, "s"

    def LoadSettings( self ):
	"""
	Load parameter values from another annotation file.
	"""

        doreadbgmodel = not (params.interactive or self.IsBGModel())
        if doreadbgmodel and self.has( 'movie' ):
            bg_img_shape = (self.movie.get_height(), self.movie.get_width())
        else:
            bg_img_shape = None
        print "loading settings from file " + self.settingsfilename
        print bg_img_shape, doreadbgmodel
        try:
            annot.LoadSettings(self.settingsfilename,
                               self.bg_imgs,
                               bg_img_shape=bg_img_shape,
                               readbgmodel=doreadbgmodel)
        except:
            print 'Could not read annotation file ' + self.settingsfilename
            raise
            
    def MacOpenFile( self, filename ):
        """Fires when a compatible file is dropped onto the app on Mac."""
        self.movie = movies.Movie( filename, interactive=params.interactive,
                                   parentframe=self.frame, open_now=True )
        self.OpenMovie( new_movie=False )

    def OpenMovie( self, new_movie=True ):
        """Attempt to open a movie given the current filename."""
        if new_movie:
            try:
                # open movie file
                self.movie = movies.Movie( self.dir, params.interactive,
                                           default_extension=self.last_movie_ext )
                if DEBUG:
                    print "Opened movie " + str(self.movie.filename)
            except ImportError: # open was cancelled
                return
            except Exception, details:
                # error messages should be handled by the movie object
                if self.has( 'movie' ):
                    if params.interactive:
                        wx.MessageBox( "Could not open the movie " + self.movie.filename,
                                       "Error", wx.ICON_ERROR|wx.OK )
                    else:
                        print "Could not open the movie " + self.movie.filename
                    print details
                    self.movie = None
                    self.ShowCurrentFrame()
                self.start_frame = 0
                return

        if not self.has( 'movie' ):
            return

        if new_movie:
            self.start_frame = 0

        # close settings windows
        if self.has( 'bg_imgs' ):
            self.OnQuitBGModel( None )
        if self.bg_window_open:
            self.OnQuitBG( None )
        if self.tracking_settings_window_open:
            self.OnQuitTrackingSettings( None )
        if self.choose_orientations_window_open:
            self.OnQuitChooseOrientations( None )
        if self.has( 'wizard' ):
            self.OnQuitTrackingWizard( None )

        params.movie_name = self.movie.filename
        self.dir = self.movie.dirname
        draw.const.filepath = self.movie.fullpath
        base, self.last_movie_ext = os.path.splitext( self.movie.filename )
        
        self.img_size = [self.movie.get_height(),self.movie.get_width()]

        # get a pointer to the "Ctraxmain" child
        if params.interactive:
            self.reset_img_wind()
            img = num.zeros((self.img_size[0],self.img_size[1]),dtype=num.uint8)
            sys.stdout.flush()
            self.img_wind.update_image_and_drawings("Ctraxmain",
                                                    img,
                                                    format="MONO8")
            sys.stdout.flush()
            self.img_wind_child = self.img_wind.get_child_canvas("Ctraxmain")
            # mouse click
            self.img_wind_child.Bind(wx.EVT_LEFT_DOWN,self.OnMouseDown)
            self.img_wind_child.Bind( wx.EVT_MOTION, self.OnMouseMoved )
            self.img_wind_child.Bind( wx.EVT_LEFT_UP, self.OnMouseUp )
            self.img_wind_child.Bind( wx.EVT_LEFT_DCLICK, self.OnMouseDoubleClick )
            
        # setup background-subtraction pieces
        self.bg_imgs = bg.BackgroundCalculator( self.movie )
        if DEBUG: print "initialized backsub data structure"

        # read settings
        if self.has( 'settingsfilename' ):
            try:
                self.LoadSettings()
            except: pass

        # open annotation file, read header if readable, read
        # tracks if user wants to load tracks
        if params.interactive:
            start_color = self.status.GetBackgroundColour()
            self.status.SetBackgroundColour( params.status_blue )
            self.status.SetStatusText( "Reading annotation from file",
                                       params.status_box )
            wx.BeginBusyCursor()
            try:
                wx.Yield() # error when called recursively, as when playing
            except: pass

        # create annotation file
        if DEBUG: print "creating/reading annotation file"
        if self.has( 'ann_filename' ): # only if sent on command line
            new_ann_filename = self.ann_filename
        else:
            if self.menu.IsChecked( xrc.XRCID("menu_settings_annprompt") ):
                prompt_title = "Choose annotation file"
            else:
                prompt_title = None
            new_ann_filename = self.get_filename_with_extension( '+.ann', prompt_title=prompt_title )
            # KB 20120109: if cancel hit, program should exit
            if new_ann_filename is None:
                self.OnQuit( None )
                exit( 2 )

        self.close_annfile() # if one is currently open
        # KB 20120109: in non-interactive mode, weird things were happening if the file exists.
        # set to overwrite file and ignore existing trajectories if in non-interactive mode
        # KB 20120111: adding option to resume tracking in non-interactive mode
        #self.ann_file = annot.AnnotationFile( new_ann_filename, self.bg_imgs, doreadtrx=(params.interactive or params.noninteractive_resume_tracking) )
        # JAB 20130116: switch to new annfiles.py
        try:
            self.ann_file = annot.AnnotationFile( new_ann_filename, self.bg_imgs, justreadheader=(not (params.interactive or params.noninteractive_resume_tracking)), bg_img_shape=(self.movie.get_height(), self.movie.get_width()) )
        except ValueError:
            s = "Could not read annotation header, not reading in %s."%new_ann_filename
            if params.feedback_enabled:
                wx.MessageBox( s, "Warning", wx.ICON_WARNING|wx.OK )
            else:
                print s

            try:
                os.remove( new_ann_filename )
            except Exception, details:
                s = "Could not delete unreadable annotation file %s:\n  %s." % (new_ann_filename, str( details ))
                if params.feedback_enabled:
                    wx.MessageBox( s, "Error", wx.ICON_ERROR|wx.OK )
                else:
                    print s

                self.movie = None
                if params.interactive:
                    self.status.SetBackgroundColour( start_color )
                    self.status.SetStatusText( "", params.status_box )
                    self.EnableControls()
                    self.UpdateStatusMovie()
                    wx.EndBusyCursor()
                return
                
            self.ann_file = annot.AnnotationFile( new_ann_filename )
        if DEBUG: print "done reading annotation file"

        # done reading annotation file
        if params.interactive:
            self.status.SetBackgroundColour( start_color )
            self.status.SetStatusText( "", params.status_box )
            wx.EndBusyCursor()
            try:
                wx.Yield() # error when called recursively, as when playing
            except: pass

        if params.interactive and len( self.ann_file ) > 0:
            self.menu.Check( xrc.XRCID("menu_playback_show_ann"), True )

        # notification for newer Ctrax version, if available
        if self.has( 'update_alert_string' ):
            if params.interactive and \
                   self.menu.IsChecked( xrc.XRCID("menu_help_updates") ):
                wx.MessageBox( self.update_alert_string,
                               "Ctrax can be updated", wx.OK )
            elif not params.interactive:
                print "============================================"
                print self.update_alert_string
                print "============================================"
            delattr( self, 'update_alert_string' )
            if self.has( 'newer_ctrax_avail' ):
                self.newest_ctrax_notified = self.newer_ctrax_avail
            if self.has( 'newer_matlab_avail' ):
                self.newest_matlab_notified = self.newer_matlab_avail

        # show movie and update GUI
        if params.interactive:
            self.OnMouseDoubleClick( None ) # zoom out
            self.ShowCurrentFrame()
            self.EnableControls()
            self.UpdateStatusMovie()
	    self.InitializeFrameSlider()
            self.OnResize()


    def OnOpen( self, evt ):
        """Movie file selection dialog."""

        old_mov = self.movie

        # open movie
        self.OpenMovie( new_movie=True )

        if self.movie is not old_mov:
            if self.menu.GetLabel( xrc.XRCID("menu_track_start") ) == const.TRACK_STOP:
                self.OnStopTracking( None ) # quit in mid-operation
            self.play_break = True # stop playback, if in progress
            self.start_frame = 0


    def OnLoadSettings( self, evt ):
        settings_file = self.get_filename_with_extension( '+.ann', "Load Settings from File" )

        if settings_file is not None:
            if self.menu.GetLabel( xrc.XRCID("menu_track_start") ) == const.TRACK_STOP:
                self.OnStopTracking( None ) # quit in mid-operation
            self.play_break = True # stop playback, if in progress
            self.settingsfilename = settings_file
            self.start_frame = 0
            self.LoadSettings()


    def OnSaveSettings( self, evt ):
        settings_file = self.get_filename_with_extension( '+.ann', "Save Settings to File" )

        if settings_file is not None:
            if params.feedback_enabled and os.access( settings_file, os.F_OK ):
                if wx.MessageBox( "This will destroy any existing annotation data in the file. The file may have been automatically backed up, but you should verify. Continue saving settings to this file anyway?", "Overwrite Data?", wx.YES_NO ) == wx.NO:
                    return

            if os.path.exists( settings_file ):
                os.remove( settings_file )
            outfile = annot.AnnotationFile( settings_file,
                                            self.bg_imgs,
                                            justreadheader=True,
                                            readbgmodel=False )
            outfile.InitializeData()


    def get_filename_with_extension( self, new_ext, prompt_title=None ):
        """Prompt user for a new filename, based on the movie filename but with a new extension.
Setting '+' as the first character of new_ext indicates that the new extension
should be appended rather than altered/replaced. If prompt_title is None, then
the new name is created and returned without user input."""
        if not self.has( 'movie' ): return None

        if new_ext[0] == '+':
            new_ext = new_ext[1:]
            replace_ext = False # append extension
        else:
            replace_ext = True # replace extension
        
        if (prompt_title is not None) and (params.feedback_enabled) and (not DEBUG):
            # prompt for new filename
            if replace_ext:
                (basename, ext) = os.path.splitext( self.movie.filename )
            else:
                basename = self.movie.filename
            new_name = basename + new_ext
            
            prompt_base, prompt_ext = os.path.splitext( new_ext )
            if prompt_ext == '':
                prompt_ext = prompt_base
            
            dlg = wx.FileDialog( self.frame, prompt_title, self.movie.dirname,
                                 new_name, "*" + prompt_ext, wx.SAVE )

            # concatenate new path name and return
            if dlg.ShowModal() == wx.ID_OK:
                save_path = os.path.join( dlg.GetDirectory(), dlg.GetFilename() )
            else:
                save_path = None

            dlg.Destroy()

        else:
            # do it without prompting
            if replace_ext:
                (basename, ext) = os.path.splitext( self.movie.fullpath )
            else:
                basename = self.movie.fullpath
            save_path = basename + new_ext
                
        return save_path
            

    def OnSave( self, evt ):
        """Choose filename to save annotation data as MAT-file."""
        # check data validity
        if len( self.ann_file ) <= 0:
            if params.interactive:
                wx.MessageBox( "No valid annotation\nexists for this movie\nor no movie is loaded.",
                               "Error", wx.ICON_ERROR|wx.OK )
            else:
                print "Not saving -- no data"
            return

        # prompt to choose orientations, if not already done
        if not self.ann_file.orientations_chosen:
            if params.interactive:
                if wx.MessageBox( "Fly orientations have not been disambiguated yet. Do it now?", "Choose orientations?", wx.YES_NO ) == wx.YES:
                    self.OnChooseOrientations()
                    while hasattr( self, 'choose_orientations' ):
                        self.choose_orientations.frame.Raise()
                        wx.Yield()
                        time.sleep( 0.1 )
            else:
                choose_orientations = chooseorientations.ChooseOrientations(self.frame,interactive=False)
                self.ann_file = choose_orientations.ChooseOrientations( self.ann_file, bg_model=self.bg_imgs )

        # get save filename
        mat_name = self.get_filename_with_extension( '.mat', "Save as MAT-file" )

        # do save
        if mat_name is not None:
            # set up to save file
            start_color = self.status.GetBackgroundColour()
            self.status.SetBackgroundColour( params.status_blue )
            self.status.SetStatusText( "writing annotation data to file", params.status_box )
            wx.BeginBusyCursor()
            wx.Yield()

            # save file
            self.ann_file.WriteMAT( self.movie, mat_name )

            # restore UI
            self.status.SetBackgroundColour( start_color )
            self.status.SetStatusText( "", params.status_box )
            wx.EndBusyCursor()
            wx.Yield()


    def OnSaveAvi( self, evt ):
        """Choose filename to save tracks as AVI-file."""
        if len( self.ann_file ) <= 0:
            if params.interactive:
                wx.MessageBox( "No valid annotation\nexists for this movie\nor no movie is loaded.",
                               "Error", wx.ICON_ERROR|wx.OK )
            else:
                print "Not saving -- no data"
            return

        dlg = wx.TextEntryDialog(self.frame,"Frames to output to AVI file: (startframe:endframe): ","Save as AVI-file","%d:%d"%(self.ann_file.firstframetracked,self.ann_file.lastframetracked))
        isgood = False
        while isgood == False:
            if dlg.ShowModal() == wx.ID_OK:
                isgood = True
                s = dlg.GetValue()
                s = s.rsplit(':')
            else:
                break
            if len(s) == 2:
                if s[0].isdigit() and s[1].isdigit():
                    framestart = int(s[0])
                    frameend = int(s[1])
                else:
                    isgood = False
                    continue
            else:
                isgood = False
                continue

        dlg.Destroy()

        if isgood == False:
            return

        avi_name = self.get_filename_with_extension( '_annot.avi', "Save as AVI file" )

        if avi_name is not None:
            if params.interactive:
                start_color = self.status.GetBackgroundColour()
                self.status.SetBackgroundColour( params.status_blue )
                self.status.SetStatusText( "writing movie of results", params.status_box )
                wx.BeginBusyCursor()
                wx.Yield()

            movies.write_results_to_avi( self.movie, self.ann_file, avi_name,
                                         framestart, frameend )

            if self.alive and params.interactive:
                self.status.SetBackgroundColour( start_color )
                self.status.SetStatusText( "", params.status_box )
                wx.EndBusyCursor()
                wx.Yield()

            print "movie exported successfully"


    def OnSaveCsv( self, evt ):
        """Choose filename to save tracks as CSV-file."""
        if len( self.ann_file ) <= 0:
            if params.interactive:
                wx.MessageBox( "No valid annotation\nexists for this movie\nor no movie is loaded.",
                               "Error", wx.ICON_ERROR|wx.OK )
            else:
                print "Not saving -- no data"
            return

        csv_name = self.get_filename_with_extension( '.csv', "Save as CSV file" )

        if csv_name is not None:
            if params.interactive:
                start_color = self.status.GetBackgroundColour()
                self.status.SetBackgroundColour( params.status_blue )
                self.status.SetStatusText( "writing plaintext results", params.status_box )
                wx.BeginBusyCursor()
                wx.Yield()

            self.ann_file.WriteCSV( csv_name )

            if self.alive and params.interactive:
                self.status.SetBackgroundColour( start_color )
                self.status.SetStatusText( "", params.status_box )
                wx.EndBusyCursor()
                wx.Yield()


    def OnSaveDiagnostics( self, evt ):
        """Choose filename to save diagnostics to."""

        diag_name = self.get_filename_with_extension( '_ctraxdiagnostics.txt',
                                                      "Save diagnostics to text file" )

        if diag_name is not None:
            start_color = self.status.GetBackgroundColour()
            self.status.SetBackgroundColour( params.status_blue )
            self.status.SetStatusText( "writing diagnostics to file",
                                       params.status_box )
            wx.BeginBusyCursor()
            wx.Yield()

            annot.WriteDiagnostics( diag_name )

            self.status.SetBackgroundColour( start_color )
            self.status.SetStatusText( "", params.status_box )
            wx.EndBusyCursor()
            wx.Yield()


    def EnableControls( self ):
        """Enable or disable GUI controls based on current state of tracker."""

	if not params.interactive:
	    return

        if self.has( 'batch' ):
            self.batch.EnableControls()

        twiddling_enabled = params.feedback_enabled and not self.tracking

	movieready = self.has( 'movie' )
        if movieready:
            issbfmf = (hasattr( self.movie, 'type' ) and self.movie.type == 'sbfmf')
        else:
            issbfmf = False

	annready = movieready and self.has( 'ann_file' ) and len( self.ann_file ) > 0

	isplaying = hasattr(self,'play_break') and not self.play_break

	self.menu.Enable( xrc.XRCID("menu_track_start"), # could be "stop"
			  movieready and (not isplaying) )

        settings_enabled = movieready and twiddling_enabled
        
        self.menu.Enable( xrc.XRCID("menu_track_writesbfmf"), settings_enabled and not issbfmf )
        self.dowritesbfmf = self.dowritesbfmf and not issbfmf
        self.menu.Check( xrc.XRCID("menu_track_writesbfmf"), self.dowritesbfmf )

	self.menu.Enable( xrc.XRCID("menu_track_resume"),
			  settings_enabled and (not isplaying) and annready \
                          and len( self.ann_file ) < self.movie.get_n_frames() )
	self.menu.Enable( xrc.XRCID("menu_track_resume_here"),
			  settings_enabled and (not isplaying) and annready )

	self.menu.Enable( xrc.XRCID("menu_load_settings"), settings_enabled )
	self.menu.Enable( xrc.XRCID("menu_save_settings"), settings_enabled )
	self.menu.Enable( xrc.XRCID("menu_settings_bg"), settings_enabled )
	self.menu.Enable( xrc.XRCID("menu_settings_bg_model"), settings_enabled )
	self.menu.Enable( xrc.XRCID("menu_settings_tracking"), settings_enabled )
        self.menu.Enable( xrc.XRCID("menu_tracking_wizard"), settings_enabled )
	self.menu.Enable( xrc.XRCID("menu_compute_background"), settings_enabled )
	self.menu.Enable( xrc.XRCID("menu_compute_shape"), settings_enabled )
        self.menu.Enable( xrc.XRCID("menu_file_save_diagnostics"), settings_enabled )
        self.framenumber_text.Enable( settings_enabled )
	self.slider.Enable( settings_enabled )
        self.frameinc_button.Enable( settings_enabled )
        self.framedec_button.Enable( settings_enabled )

        saving_enabled = annready and twiddling_enabled
	self.menu.Enable( xrc.XRCID("menu_playback_show_ann"), saving_enabled )
	self.menu.Enable( xrc.XRCID("menu_file_export"), saving_enabled )
	self.menu.Enable( xrc.XRCID("menu_file_save_avi"), saving_enabled )
	self.menu.Enable( xrc.XRCID("menu_choose_orientations"), saving_enabled )
        self.menu.Enable( xrc.XRCID("menu_analyze_plottraj"), saving_enabled )
        self.menu.Enable( xrc.XRCID("menu_analyze_plotvel"), saving_enabled )
        self.menu.Enable( xrc.XRCID("menu_analyze_histpos"), saving_enabled )
        self.menu.Enable( xrc.XRCID("menu_analyze_histspeed"), saving_enabled )
        self.menu.Enable( xrc.XRCID("menu_analyze_histdtheta"), saving_enabled )


    def InitializeFrameSlider(self):
        self.slider.SetThumbPosition( self.start_frame )
        self.slider.SetScrollbar( self.start_frame,1,self.movie.get_n_frames()-1,100 )
        self.framenumber_text.SetValue( "%05d"%(self.movie.get_n_frames()) )
        

    def UpdateStatusMovie( self ):
        """Update status bar with movie filename."""
        try:
            if not self.has( 'movie' ) or len( self.movie.filename ) == 0:
                self.status.SetStatusText( "[no file loaded]", params.file_box )
            elif len( self.movie.filename ) < params.file_box_max_width:
                self.status.SetStatusText( self.movie.filename, params.file_box )
            else:
                self.status.SetStatusText( "..." + os.sep + self.movie.filename, params.file_box )
        except (TypeError, AttributeError): pass


    def ShowCurrentFrame( self ):
        """Grab current frame, draw on it, and display in GUI.
        Also update zoom-ellipse windows, if present."""
        if not params.interactive: return
        if not self.alive: return
        if not self.has( 'movie' ): return
        if self.start_frame < 0: return

        # get frame
        try:
            frame, self.last_timestamp = self.movie.get_frame( self.start_frame )
            if num.isnan(self.last_timestamp):
                self.last_timestamp = float(self.start_frame) / float(params.DEFAULT_FRAME_RATE)
        except movies.NoMoreFramesException:
            self.start_frame = min(self.start_frame,self.movie.get_n_frames()-1)
            self.slider.SetScrollbar( self.start_frame,1,self.movie.get_n_frames()-1,100 )
            return
        except IndexError: # framenumber out of range
            return

        # set frame number display
        self.framenumber_text.SetValue( "%05d"%(self.start_frame) )

        # dim frame
        if self.menu.IsChecked( xrc.XRCID("menu_playback_dim") ):
            frame = frame / 2

        # annotate image
        dodrawann = len( self.ann_file ) > 0 and \
            self.start_frame >= self.ann_file.firstframetracked and \
            self.start_frame <= self.ann_file.lastframetracked

        frame8 = imagesk.double2mono8(frame,donormalize=False)
        height, width = frame8.shape

        # choose correct region of interest
        if self.has( 'zoom_drag_roi' ):
            left = int( round( width*self.zoom_drag_roi[0] ) )
            top = int( round( height*self.zoom_drag_roi[1] ) )
            right = int( round( width*self.zoom_drag_roi[2] ) )
            bottom = int( round( height*self.zoom_drag_roi[3] ) )
            frame8 = frame8[top:bottom,left:right]

            # resize the image
            resizew = float( width )/max( float( frame8.shape[1] ), 0.1 )
            resizeh = float( height )/max( float( frame8.shape[0] ), 0.1 )
            self.zoom_drag_roi_scale = min( resizew, resizeh )
            try:
                new_frame8 = imresize( frame8, self.zoom_drag_roi_scale )
            except ValueError:
                return # "tile cannot extend outside image", seems to be harmless
            
            # fill in with gray at margins
            frame8 = num.ones( frame.shape, dtype=num.uint8 )*127
            #x = (frame8.shape[1] - new_frame8.shape[1])/2
            #y = (frame8.shape[0] - new_frame8.shape[0])/2
            # centering is a lot of work for subsequent clicks
            frame8[:new_frame8.shape[0],:new_frame8.shape[1]] = new_frame8
            
        # draw bounding box for drag rectangle
        lines_to_draw = []
        line_colors = []
        if self.zoom_dragging:
            lines_to_draw.extend( [(width*self.zoom_drag_origin[0],
                                    height*self.zoom_drag_origin[1],
                                    width*self.zoom_drag_origin[0],
                                    height*self.zoom_drag_current[1]),
                                   (width*self.zoom_drag_origin[0],
                                    height*self.zoom_drag_current[1],
                                    width*self.zoom_drag_current[0],
                                    height*self.zoom_drag_current[1]),
                                   (width*self.zoom_drag_current[0],
                                    height*self.zoom_drag_current[1],
                                    width*self.zoom_drag_current[0],
                                    height*self.zoom_drag_origin[1]),
                                   (width*self.zoom_drag_current[0],
                                    height*self.zoom_drag_origin[1],
                                    width*self.zoom_drag_origin[0],
                                    height*self.zoom_drag_origin[1])] )
            line_colors.extend( [params.zoom_drag_rectangle_color,
                                 params.zoom_drag_rectangle_color,
                                 params.zoom_drag_rectangle_color,
                                 params.zoom_drag_rectangle_color] )

        # draw annotation
        if self.menu.IsChecked( xrc.XRCID("menu_playback_show_ann") ) and dodrawann:
            # first frame of tail of trajectory
            tailframe = max( self.ann_file.firstframetracked,
                             self.start_frame - params.tail_length )
            dataframes = self.ann_file[tailframe:self.start_frame + 1]

            # update small ellipse windows
            if self.menu.IsChecked( xrc.XRCID("menu_settings_zoom") ):
                self.zoom_window.SetData(dataframes[-1],frame)
                self.zoom_window.Redraw()

            ellipses = dataframes[-1]
            old_pts = []
            for dataframe in dataframes:
                these_pts = []
                for ellipse in dataframe.itervalues():
                    these_pts.append( (ellipse.center.x, ellipse.center.y, ellipse.identity) )
                old_pts.append( these_pts )

            # draw on image
            linesegs = ell.annotate_image( ellipses, old_pts )
            (linesegs,linecolors) = imagesk.separate_linesegs_colors(linesegs)
            lines_to_draw.extend( linesegs )
            line_colors.extend( linecolors )

            self.num_flies_text.SetValue( "N. Flies: %02d"%len(ellipses) )
        else:
            self.num_flies_text.SetValue( "" )

        # scale the drawings
        if self.has( 'zoom_drag_roi' ):
            for si in range( len( lines_to_draw ) ):
                orig_seg = lines_to_draw[si]
                new_seg = ((orig_seg[0] - left)*self.zoom_drag_roi_scale,
                           (orig_seg[1] - top)*self.zoom_drag_roi_scale,
                           (orig_seg[2] - left)*self.zoom_drag_roi_scale,
                           (orig_seg[3] - top)*self.zoom_drag_roi_scale)
                lines_to_draw[si] = new_seg

        # draw
        self.img_wind.update_image_and_drawings( 'Ctraxmain',
                                                 frame8,
                                                 format="MONO8",
                                                 linesegs=lines_to_draw,
                                                 lineseg_colors=line_colors,
                                                 lineseg_widths=[params.ellipse_thickness]*len( lines_to_draw ) )
        self.img_wind.Refresh( eraseBackground=False )

        # update the slider
        self.slider.SetThumbPosition( self.start_frame )
        self.frameinc_button.Enable( self.start_frame < self.movie.get_n_frames() - 1 )
        self.framedec_button.Enable( self.start_frame > 0 )


    def OnSlider( self, evt ):
        """Frame slider callback. Display new frame."""
        # tone down the duplicated events as much as possible (Windows, I mean you)
        new_fr = self.slider.GetThumbPosition()
        if new_fr != self.start_frame:
            try:
                wx.Yield()
            except:
                pass
            else:
                self.play_break = True
                self.start_frame = new_fr
                self.ShowCurrentFrame()
            finally:
                evt.Skip()
                

    def OnResize( self, evt=None ):
        """Window resized. Repaint in new window size."""

        if hasattr( self, 'img_size' ):
            top_sizer = self.frame.GetSizer()
            panel_item = top_sizer.GetItem( self.img_panel )
            panel_item.SetFlag( panel_item.GetFlag() | wx.SHAPED )
            panel_item.SetRatio( float( self.img_size[1] )/self.img_size[0] )

        self.frame.Layout()

        try:
            # redraw
            self.ShowCurrentFrame()
            # scale slider to new width
            button_size = self.frameinc_button.GetRect().GetWidth()
            new_size = wx.Size( self.img_wind.GetRect().GetWidth() - 2*button_size,
                                self.slider.GetRect().GetHeight() )
            self.slider.SetMinSize( new_size )
            self.slider.SetSize( new_size )
            new_pos = wx.Point( self.img_panel.GetPosition().x + button_size,
                                self.slider.GetPosition().y )
            self.slider.SetPosition( new_pos )
            new_pos = wx.Point( self.img_panel.GetPosition().x,
                                self.framedec_button.GetPosition().y )
            self.framedec_button.SetPosition( new_pos )
            new_pos.x = self.img_panel.GetPosition().x + new_size.width + button_size
            self.frameinc_button.SetPosition( new_pos )
            # scale movie-name box
            const.file_box_max_width = int(float(self.img_wind.GetRect().GetWidth())/11.)
            self.UpdateStatusMovie()
        except AttributeError: pass # during initialization


    def close_annfile( self ):
        """Close annotation file, as when opening a new movie or quitting."""
        if self.has( 'ann_file' ):
            self.ann_file.close()
            
            if self.dowritesbfmf:
                self.ann_file.CopyToSBFMF()

        
    def OnQuit( self, evt ):
        """Quit selected (or window closing). Stop threads and close window."""

        # stop tracking
        if self.menu.GetLabel( xrc.XRCID("menu_track_start") ) == const.TRACK_STOP:
            self.OnStopTracking( None ) # quit in mid-operation
        self.play_break = True

        try:
            self.close_annfile()
        except Exception, details:
            print "error closing annotation: %s"%details

        # write user settings
        self.WriteUserfile()

        # kill
        self.alive = False
        self.frame.Destroy()


    def OnStopTracking( self, evt=None ):
        """Stop button pressed. Stop threads."""

        self.StopThreads() # located in algorithm.py
        params.enable_feedback( True )
        if self.has( 'batch' ):
            self.batch.executing = False

        if self.dowritesbfmf and self.movie.writesbfmf_isopen():
            self.movie.writesbfmf_close(self.start_frame)

        # set tracking flag
        self.tracking = False


    def OnStartTrackingMenu( self, evt ):
        """Start button pressed. Begin tracking."""
        if self.menu.GetLabel( xrc.XRCID("menu_track_start") ) == const.TRACK_STOP:
            # stop tracking
            self.OnStopTracking()
        else:
            self.OnStartTracking(evt)


    def OnWriteSBFMF( self, evt=None ):
        """Set SBFMF filename. Use user input if running interactively."""
        self.dowritesbfmf = False

        if self.has( 'movie' ) and \
               self.menu.IsChecked( xrc.XRCID("menu_track_writesbfmf") ):
            self.writesbfmf_filename = self.get_filename_with_extension( '.sbfmf' )
            self.dowritesbfmf = True


    def OnPlayButton( self, evt ):
        self.OnStartPlayback()


    def OnStopButton(self,evt):
        if self.tracking:
            self.OnStopTracking()
        else:
            self.OnStopPlayback()


    def OnStartTracking(self,evt=None):

        # check for bg model
        if not self.CheckForBGModel():
            return

        if not self.CheckForShapeModel():
            return

        # will data be lost?
        if evt is not None and params.interactive and self.has( 'ann_file' ) and \
                len( self.ann_file ) > 0 and not DEBUG:
           if evt.GetId() == xrc.XRCID("menu_track_start"): 
               msgtxt = 'Frames %d to %d have been tracked.\nErase these results and start tracking over?' % (self.ann_file.firstframetracked,self.ann_file.lastframetracked)
               if wx.MessageBox( msgtxt, "Erase trajectories and start tracking?", wx.OK|wx.CANCEL ) == wx.CANCEL:
                   return
           elif evt.GetId() == xrc.XRCID("menu_track_resume_here"):
               if self.ann_file.lastframetracked >= self.start_frame:
                   msgtxt = 'Frames %d to %d have been tracked.\nRestarting tracking at frame %d will cause old trajectories from %d to %d to be erased.\nErase these results and restart tracking in the current frame?' % (self.ann_file.firstframetracked, self.ann_file.lastframetracked, self.start_frame, self.start_frame, self.ann_file.lastframetracked)
                   if wx.MessageBox( msgtxt, "Erase trajectories and start tracking?", wx.OK|wx.CANCEL ) == wx.CANCEL:
                       return
        # end check for trajectory overwriting

        # set tracking flag
        self.tracking = True

        # update toolbar functions
        self.UpdateToolBar('started')

        self.menu.SetLabel( xrc.XRCID("menu_track_start"), const.TRACK_STOP )
        self.menu.Check( xrc.XRCID("menu_playback_show_ann"), True )

	self.EnableControls()

        wx.Yield() # refresh GUI

        # crop data
        if evt is not None and evt.GetId() == xrc.XRCID("menu_track_resume"):
            # if resuming tracking, we will keep the tracks from 
            # frames firstframetracked to lastframetracked-1 
            # (remove last frame in case writing the last frame 
            # was interrupted)
            self.start_frame = self.ann_file.lastframetracked
            if DEBUG: print "start_frame = " + str(self.start_frame)
            if DEBUG: print "cropping annotation file to frames %d through %d"%(self.ann_file.firstframetracked,self.ann_file.lastframetracked-1)
            self.ann_file.InitializeData( self.ann_file.firstframetracked,
                                          self.ann_file.lastframetracked - 1,
                                          bg_model=self.bg_imgs )
            
            # restart writing to the sbfmf
            if self.dowritesbfmf:
                try:
                    self.movie.writesbfmf_restart(self.start_frame,self.bg_imgs,
                                                  self.writesbfmf_filename)
                except Exception, details:
                    self.abort_sbfmf_writing( details )

        elif evt is not None and evt.GetId() == xrc.XRCID("menu_track_resume_here"):
	    # if resuming here, then erase parts of track after current frame

            # the latest possible frame to start tracking on is 
            # lastframetracked
            if self.start_frame > self.ann_file.lastframetracked:
                print "Restarting tracking at frame %d (current frame > last frame tracked)"%self.ann_file.lastframetracked
                self.start_frame = self.ann_file.lastframetracked

            # crop to the frames before the current frame
            self.ann_file.InitializeData( self.ann_file.firstframetracked,
                                          self.start_frame - 1,
                                          bg_model=self.bg_imgs )
            
            # restart writing to the sbfmf
            if self.dowritesbfmf:
                try:
                    self.movie.writesbfmf_restart(self.start_frame,self.bg_imgs,
                                                  self.writesbfmf_filename)
                except Exception, details:
                    self.abort_sbfmf_writing( details )

        else:
            # start(over) tracking

            self.start_frame = 0
            params.start_frame = self.start_frame

            # empty annotations
            self.ann_file.InitializeData( self.start_frame, bg_model=self.bg_imgs )
            if self.dowritesbfmf:
                # open an sbfmf file if necessary
                self.movie.writesbfmf_start( self.bg_imgs, self.writesbfmf_filename )

        self.Track() # located in algorithm.py

        if self.alive:
            self.OnStopTracking()

            # update toolbar function
            self.UpdateToolBar('stopped')
	    self.menu.SetLabel( xrc.XRCID("menu_track_start"), const.TRACK_START )

	    self.EnableControls()


    def abort_sbfmf_writing( self, exception ):
        msgtxt = 'Could not restart writing sbfmf; file %s was unreadable. Not writing sbfmf.'%self.writesbfmf_filename
        if params.interactive:
            wx.MessageBox( msgtxt, "Warning", wx.ICON_WARNING|wx.OK )
        else:
            print msgtxt
        print "SBFMF restart error was:", str( exception )
        self.dowritesbfmf = False
        self.menu.Check( xrc.XRCID("menu_track_writesbfmf"), False)
        self.movie.writesbfmf_close( self.start_frame )
        

    def OnComputeBg(self,evt=None):

        if params.interactive:
            start_color = self.status.GetBackgroundColour()
            self.status.SetBackgroundColour( params.status_green )
            self.status.SetStatusText( "calculating background", params.status_box )

            success = self.bg_imgs.OnCalculate( parent=self.frame )
            
            self.status.SetBackgroundColour( start_color )
            self.status.SetStatusText( "", params.status_box )

        else:
            success = self.bg_imgs.OnCalculate()

        return success


    def OnComputeShape(self,evt=None):
        if not self.CheckForBGModel(): return

        if params.interactive:
            start_color = self.status.GetBackgroundColour()
            self.status.SetBackgroundColour( params.status_red )
            self.status.SetStatusText( "calculating shape", params.status_box )
            wx.BeginBusyCursor()
            wx.Yield()

            success = ell.est_shape( self.bg_imgs, self.frame )
            
            wx.EndBusyCursor()
            self.status.SetBackgroundColour( start_color )
            self.status.SetStatusText( "", params.status_box )
            
        else:
            success = ell.est_shape( self.bg_imgs )

        return success


    def OnStopPlayback( self, evt=None ):
        # pause playback
        self.play_break = True

        # update toolbar function
        self.UpdateToolBar('stopped')

        # change menu items
	self.EnableControls()


    def OnStartPlayback( self, evt=None ):
        """Begin playback."""
        if not self.has( 'movie' ): return

        # update toolbar function
        self.UpdateToolBar('started')

        self.play_break = False

        # change menu items
	self.EnableControls()

        # start playback
        self.start_frame += 1 # don't play current frame again
        if self.start_frame >= self.movie.get_n_frames():
            self.start_frame = 0 # start over
            self.last_timestamp = self.movie.get_some_timestamps( 0, 1 )[0]
        self.play_start_stamp = self.last_timestamp
        self.play_start_time = time.time()
        while self.start_frame < self.movie.get_n_frames():
            # show current frame
            self.slider.SetThumbPosition( self.start_frame )
            self.ShowCurrentFrame()
            wx.Yield()
            if self.play_break: break

            # calculate which frame to show next
            # test actual and movie elapsed time
            actual_time = max( time.time() - self.play_start_time, 0.05 )
            movie_time = max( self.last_timestamp - self.play_start_stamp, 0.05 )
            # the ratio of these two should equal the play_speed...
            if movie_time / actual_time > self.play_speed:
                # more movie time has elapsed than real time, so slow down
                time.sleep( movie_time - actual_time*self.play_speed )
                self.start_frame += 1
            else:
                # more actual time has elapsed than movie time, so skip frames
                self.start_frame += int(round( actual_time*self.play_speed / movie_time ))

        if self.start_frame >= self.movie.get_n_frames():
            self.start_frame = self.movie.get_n_frames() - 1
            self.slider.SetScrollbar( self.start_frame, 1,
                                      self.movie.get_n_frames() - 1, 100 )

        if self.alive:
            self.OnStopPlayback()

    def OnSpeedUpButton( self, evt ):
        if self.tracking:
            self.OnSpeedUpTracking()
        else:
            self.OnChangePlaybackSpeed(evt)

    def OnSlowDownButton( self, evt ):
        if self.tracking:
            self.OnSlowDownTracking()
        else:
            self.OnChangePlaybackSpeed(evt)

    def OnRefreshButton( self, evt ):
        self.request_refresh = True

    def OnSpeedUpTracking(self):

        self.framesbetweenrefresh = max(1,self.framesbetweenrefresh-1)
        self.set_rate_text( use_refresh=True )

    def OnSlowDownTracking(self):

        self.framesbetweenrefresh += 1
        self.set_rate_text( use_refresh=True )

    def OnChangePlaybackSpeed(self,evt=None):
        """Change playback speed."""
        
        if evt.GetId() == self.speedup_id:
            self.play_speed *= 2
        elif evt.GetId() == self.slowdown_id:
            self.play_speed /= 2
        self.play_speed = min( self.play_speed, 32.0 )
        self.play_speed = max( self.play_speed, 1/32. )
        self.set_rate_text( use_refresh=False )

        # reset timers
        if self.has( 'movie' ):
            self.play_start_stamp = self.last_timestamp
            self.play_start_time = time.time()


    def OnBatch( self, evt ):
        """Open batch processing window."""
        if self.batch is not None and self.batch.is_showing:
            self.batch.frame.Raise()
            return

        # open selector window
        if self.batch is None:
            if self.has( 'movie' ):
                self.batch = batch.BatchWindow( self.frame,
                                                self.dir,
                                                self.last_movie_ext,
                                                self.movie )
            else:
                self.batch = batch.BatchWindow( self.frame,
                                                self.dir,
                                                self.last_movie_ext )
        else: # not none but not showing, either
            self.batch.ShowWindow( self.frame )

        # bind callbacks
        self.batch.frame.Bind( wx.EVT_SIZE, self.OnBatchResize )
        self.batch.frame.Bind( wx.EVT_MOVE, self.OnBatchResize )
        self.batch.frame.Bind( wx.EVT_BUTTON, self.OnBatchExecute, id=xrc.XRCID("button_execute") )

        # set window position from memory
        if self.last_batch_pos is not None:
            batch_pos = self.constrain_window_to_screen( self.last_batch_pos, self.last_batch_size )
            self.batch.frame.SetPosition( batch_pos )
            self.batch.frame.SetSize( self.last_batch_size )

        # set "save SBFMF" checkbox based on own state
        self.batch.sbfmf_checkbox.SetValue( self.menu.IsChecked( xrc.XRCID("menu_track_writesbfmf") ) )


    def OnBatchResize( self, evt ):
        """Batch window was moved or resized, remember new location."""
        evt.Skip()
        try:
            self.last_batch_size = self.batch.frame.GetSize()
            self.last_batch_pos = self.batch.frame.GetPosition()
        except AttributeError: pass # during initialization


    def OnBatchExecute( self, evt ):
        """Begin executing batch processing."""
        if len( self.batch.file_list ) == 0:
            return
        
        if params.interactive and wx.MessageBox( "Execute batch processing now?", "Execute?", wx.YES_NO ) == wx.NO:
            return

        if self.batch.should_load_settings():
            if not self.has( 'movie' ):
                self.movie = movies.Movie( self.batch.file_list[0]['filename'],
                                           interactive=False,
                                           open_now=False )
            settings_file = self.get_filename_with_extension( '+.ann', "Load Settings from File" )
            if settings_file is None:
                return
        else:
            settings_file = None

        # disable GUI
        params.enable_feedback( False )
        self.menu.SetLabel( xrc.XRCID("menu_track_start"), const.TRACK_STOP )
        self.EnableControls()

        if not self.menu.IsEnabled( xrc.XRCID("menu_playback_show_ann") ):
            # assume it's only not checked because this movie isn't annotated
            self.menu.Check( xrc.XRCID("menu_playback_show_ann"), True )
        wx.Yield() # refresh GUI

        write_sbfmfs_init = self.dowritesbfmf
        write_sbfmfs = self.batch.sbfmf_checkbox.IsChecked()

        if params.batch_autodetect_arena is None:
            params.do_set_circular_arena = False
        else:
            params.do_set_circular_arena = params.batch_autodetect_arena

        self.batch.executing = True

        # run batch
        batch_list = self.batch.file_list[:]
        for movie_data in batch_list:
            # open movie
            self.movie = movies.Movie( movie_data['filename'],
                                       interactive=False )
            self.OpenMovie( new_movie=False )
            self.start_frame = movie_data['startframe']
            self.last_frame = movie_data['endframe']

            if write_sbfmfs and self.movie.type != 'sbfmf':
                self.dowritesbfmf = True
                self.writesbfmf_filename = self.get_filename_with_extension( '.sbfmf' )
            else:
                self.dowritesbfmf = False

            # load settings
            if settings_file is not None:
                try:
                    annot.LoadSettings( settings_file,
                                        self.bg_imgs,
                                        bg_img_shape=(self.movie.get_height(),
                                                      self.movie.get_width()),
                                        readbgmodel=True )
                except:
                    print 'Could not read settings file', settings_file
                    raise

                if params.batch_autodetect_arena == True and \
                       params.do_set_circular_arena == False:
                    # arena settings were read, but we won't be autodetecting
                    params.do_set_circular_arena = True

            # do tracking
            if self.menu.IsEnabled( xrc.XRCID("menu_track_start") ):
                self.tracking = True
                self.UpdateToolBar( 'started' )
                wx.Yield()

                self.DoAll()

                if self.dowritesbfmf:
                    self.ann_file.CopyToSBFMF()

                if self.alive:
                    self.UpdateToolBar( 'stopped' )
                    wx.Yield()
                self.tracking = False

                self.last_frame = num.inf

            # bail out if "stop" was pressed
            if not self.batch.executing: break

            # remove file from list
            if len( self.batch.file_list ) > 0:
                self.batch.file_list.pop( 0 )
                if self.batch.is_showing:
                    self.batch.list_box.Set( self.batch.file_list )

        self.batch.executing = False
        self.dowritesbfmf = write_sbfmfs_init

        if self.alive:
            # re-enable GUI
            params.enable_feedback( True )

            self.menu.SetLabel( xrc.XRCID("menu_track_start"), const.TRACK_START )
            self.EnableControls()


    def OnCheckBatchStats( self, evt ):
        """Batch statistics box checked. Force re-read of annotation data."""
        self.batch_data = None # currently irrelevant


    def OnHelp( self, evt ):
        """Help requested. Popup box with website."""
        wx.MessageBox( "Documentation available at\nhttp://ctrax.sourceforge.net", "Help", wx.OK )

    def OnAbout( self, evt ):
        """About box requested."""
        wx.AboutBox( const.info )

def main():
    args = (0,)
    kw = {}
    if (not int(os.environ.get('CTRAX_NO_REDIRECT','0'))):
        # redirect to a window
        args = ()
        kw = dict(redirect=True)

    # for debugging only: reseed the random number generator at 0 for repeatable behavior
    if DEBUG_REPEATABLE_BEHAVIOR:
        num.random.seed(0)

    app = CtraxApp( *args, **kw )
    app.MainLoop()

if __name__ == '__main__':
    #import hotshot
    #prof = hotshot.Profile("hotshot_ctrax_stats")
    #prof.runcall(main)
    #prof.close()
    main()




import copy
import sys

import numpy as num
from wx import AboutDialogInfo

from version import DEBUG, __version__

class ShapeParams:
    def __init__(self,major=0,minor=0,area=0,ecc=0):
        self.major = major
        self.minor = minor
        self.area = area
        self.ecc = ecc

    def copy(self):
        return ShapeParams(self.major,self.minor,self.area,self.ecc)

    def __print__(self):
        return 'major = %.2f, minor = %.2f, area = %.2f, ecc = %.2f'%(self.major,self.minor,self.area,self.ecc)

    def __repr__( self ): return self.__print__()

    def __str__( self ): return self.__print__()

    def __eq__(self,other):
        for i,j in self.__dict__.iteritems():
            if not hasattr(other,i):
                return False
            if not (j == other.__dict__[i]):
                return False
        for i,j in other.__dict__.iteritems():
            if not hasattr(self,i):
                return False
            if not (j == self.__dict__[i]):
                return False
        return True

def averageshape(shape1,shape2):
    shape3 = ShapeParams()
    shape3.major = (shape1.major+shape2.major)/2.
    shape3.minor = (shape1.minor+shape2.minor)/2.
    shape3.area = (shape1.area+shape2.area)/2.
    shape3.ecc = (shape1.ecc+shape2.ecc)/2.
    return shape3

class Grid:
    def __init__(self):
        self.X = None
        self.Y = None
        self.X2 = None
        self.Y2 = None
        self.XY = None
    def setsize(self,sz):
        [self.Y,self.X] = num.mgrid[0:sz[0],0:sz[1]]
        self.X = self.X.astype( num.int64 )
        self.Y = self.Y.astype( num.int64 )
        self.Y2 = self.Y**2
        self.X2 = self.X**2
        self.XY = self.X*self.Y
    def __eq__(self,other):
        return True

class Parameters:
    def __init__(self):

        self.DOBREAK = False

        # set constants

        # for timing various parts of the code
        self.last_time = 0

        # default frame rate
        self.DEFAULT_FRAME_RATE = 25.

        # for fast computation of weighted region props
        self.GRID = Grid()

        # max frac above-threshold points in a frame
        self.max_n_points_ratio = 1./250.
        # for displaying zoom windows
        self.id_spinner_width = 50
        # palettes
        self.normal_palette = [[255,0,0],     # red
                               [0,255,0],     # green
                               [0,0,255],     # blue
                               [255,0,255],   # magenta
                               [255,255,0],   # yellow
                               [0,255,255],   # cyan
                               [255,127,127], # light red
                               [127,255,127], # light green
                               [127,127,255], # light blue
                               [255,127,255], # light magenta
                               [255,255,127], # light yellow
                               [127,255,255]] # light cyan
        # colorblind-friendly palette from
        # http://jfly.iam.u-tokyo.ac.jp/color/index.html
        self.colorblind_palette = [[230,159,0],   # orange
                                   [86,180,233],  # sky blue
                                   [0,158,115],   # blue-green
                                   [240,228,66],  # yellow
                                   [0,114,178],   # blue
                                   [213,94,0],    # vermillion
                                   [204,121,167]] # red-purple

        self.zoom_drag_rectangle_color = (1, 0.6, 0, 1)

        # background constants
        self.SHOW_RAW_FRAME = -1
        self.SHOW_BACKGROUND = 0
        self.SHOW_DISTANCE = 1
        self.SHOW_THRESH = 2
        self.SHOW_NONFORE = 3
        self.SHOW_DEV = 4
        self.SHOW_CC = 5
        self.SHOW_ELLIPSES = 6
        self.SHOW_EXPBGFGMODEL_LLR = 7
        self.SHOW_EXPBGFGMODEL_ISBACK = 8
        self.SHOW_EXPBGFGMODEL_BGMU = 9
        self.SHOW_EXPBGFGMODEL_FGMU = 10
        self.SHOW_EXPBGFGMODEL_BGSIGMA = 11
        self.SHOW_EXPBGFGMODEL_FGSIGMA = 12
        self.SHOW_EXPBGFGMODEL_FRACFRAMESISBACK = 13
        self.SHOW_EXPBGFGMODEL_MISSINGDATA = 14
        
        self.BG_SHOW_STRINGS = ['Background Image',
                                'Distance from Background',
                                'Foreground/Background Classification',
                                'Background-Only Areas',
                                'Normalization Image',
                                'Connected Components',
                                'Ellipse Fits']
        self.EXPBGFGMODEL_SHOW_STRINGS = ['Prior Log-Likelihood Ratio',
                                          'Use in Background Model',
                                          'Prior Background Mean Px Intensity',
                                          'Prior Foreground Mean Px Intensity',
                                          'Prior Background Std Px Intensity',
                                          'Prior Foreground Std Px Intensity',
                                          'Frac Frames in Bg Model',
                                          'Bg Missing Data']
        self.SHOW_EXPBGFGMODEL = [self.SHOW_EXPBGFGMODEL_LLR,self.SHOW_EXPBGFGMODEL_ISBACK,
                                  self.SHOW_EXPBGFGMODEL_BGMU,self.SHOW_EXPBGFGMODEL_FGMU,
                                  self.SHOW_EXPBGFGMODEL_BGSIGMA,self.SHOW_EXPBGFGMODEL_FGSIGMA,
                                  self.SHOW_EXPBGFGMODEL_FRACFRAMESISBACK,self.SHOW_EXPBGFGMODEL_MISSINGDATA]
        
        self.ALGORITHM_MEDIAN = 0
        self.ALGORITHM_MEAN = 1

        # display stuff
        self.status_green = "#66FF66"
        self.status_blue = "#AAAAFF"
        self.status_red = "#FF6666"
        self.status_yellow = "#FFFF66"
        self.wxvt_bg = '#DDFFDD'

        self.MAXDSHOWINFO = 10 # maximum allowed click distance from selection
        self.DRAW_MOTION_SCALE = 10.

        ## User interaction mode ###
        self.interactive = True # whether Ctrax was started in interactive mode or not
        self.noninteractive_resume_tracking = False # in non-interactive mode, if we read in trajectories, should we resume tracking
        self.feedback_enabled = True # whether user feedback is currently enabled
        self.app_instance = None # pointer to the global Application

        # set parameters to default values
        self.start_frame = 0

        # Number of identities assigned so far
        self.nids = long( 0 )

        self.version = 0

        ## Movie Parameters ###

        # number of frames in the movie
        self.n_frames = 0
        # size of a frame
        self.movie_size = (0,0)
        # number of pixels in a frame
        self.npixels = 0
        # movie object
        self.movie = None
        self.movie_name = ''
        self.annotation_movie_name = ''

        self.movie_index_transpose = True

        ## Background Estimation Parameters ###

        # maximum number of pixels (should = bytes?) to allocate for temporary storage while computing the bg median
        # equivalent to 100 x (480x640) images
        self.bg_median_maxbytesallocate = 30720000
        # Background Subtraction Parameters

        # homomorphic filtering constants
        # these defaults are based on a painstaking and probably
        # inaccurate parameterization, but they're a good starting point
        self.hm_cutoff = 0.35
        self.hm_boost = 2
        self.hm_order = 2
        # number of standard deviations to threshold background at
        self.n_bg_std_thresh = 20.
        # do hysteresis
        self.n_bg_std_thresh_low = 10.
        # minimum number of standard deviations for background
        self.bg_std_min = 1.
        self.bg_std_max = 10.
        # if background intensity is greater than min_nonarena, then it
        # is not an area flies can be in, so don't allow foreground in
        # these areas
        self.min_nonarena = 256.
        # if background intensity is less than max_nonarena, then it
        # is not an area flies can be in, so don't allow foreground in
        # these areas
        self.max_nonarena = -1.
        
        # regions of interest
        self.roipolygons = []
        
        # location of arena
        self.arena_center_x = None
        self.arena_center_y = None
        self.arena_radius = None
        self.arena_edgethresh = None
        self.do_set_circular_arena = True

        # search space for arena circle
        self.min_arena_center_x = .4
        self.max_arena_center_x = .6
        self.min_arena_center_y = .4
        self.max_arena_center_y = .6
        self.min_arena_radius = .25
        self.max_arena_radius = .5

        # batch processing & auto-detecting arena
        self.batch_autodetect_arena = True
        self.batch_autodetect_shape = True
        self.batch_autodetect_bg_model = True

        # morphology
        self.do_use_morphology = False
        self.opening_radius = 0
        self.closing_radius = 0

        ## Shape Parameters ###
        
        # upper bounds on shape
        self.maxshape = ShapeParams(9999.,9999.,9999.,1.)
        # lower bounds on shape
        self.minshape = ShapeParams(1.,1.,1.,0.)
        self.meanshape = ShapeParams(2.64,3.56,40.25,0.98)
        # foreground background thresh set to minbackthresh when trying to increase target area
        self.minbackthresh = 1.
        # maximum number of clusters to split a foreground connected component into during the forward pass
        self.maxclustersperblob = 5
        # maximum penalty for merging together two ccs
        self.maxpenaltymerge = 40
        # maximum area of deleted target
        self.maxareadelete = 5
        # minimum area of ignored connected components
        self.minareaignore = 2500
        # max n objects per frame to return
        self.max_n_clusters = 100 
        # number of frames used to compute shape bounds
        self.n_frames_size = 50
        # number of standard deviations from mean for upper and lower shape bounds
        self.n_std_thresh = 4.

        ## Motion model parameters ###
        
        # weight of angle in distance measure
        self.ang_dist_wt = 100.
        # maximum distance a fly can move between frames
        self.max_jump = 100.
        # KB 20120109: maximum distance a fly can move between frames and end up within a split connected component
        self.max_jump_split = -1.
        # minimum distance to be called a jump
        self.min_jump = 50.
        # dampening constant
        self.dampen = 0. # weighting term used in cvpred()
        self.angle_dampen = 0.5 

        # weight of velocity angle in choosing orientation mod 2pi
        self.velocity_angle_weight = .05
        self.max_velocity_angle_weight = .25

        ## Fix errors parameters ###
        
        self.do_fix_split = True
        self.do_fix_merged = True
        self.do_fix_spurious = True
        self.do_fix_lost = True
        self.lostdetection_length = 50
        self.lostdetection_distance = 100.
        self.spuriousdetection_length = 50
        self.mergeddetection_distance = 20.
        self.mergeddetection_length = 50
        self.splitdetection_cost = 40.
        self.splitdetection_length = 50
        self.maxdcentersextra = 3.
        self.bigboundingboxextra = 2.
        # maximum number of frames to buffer for faster hindsight
        self.maxnframesbuffer = 100

        ## Drawing/Display parameters ###
        # thickness of lines of drawn ellipses
        if 'darwin' in sys.platform:
            self.ellipse_thickness = 1
        else:
            self.ellipse_thickness = 2
        # palette of colors to use
        self.use_colorblind_palette = False
        # colors
        self.colors = self.normal_palette

        self.tail_length = 10

        self.status_box = 0
        self.file_box = 1
        self.file_box_max_width = 40

        # computing prior background/foreground models

        # number of frames to sample per video
        self.prior_nframessample = 100

        self.expbgfgmodel_filename = None
        self.use_expbgfgmodel = False
        
        # threshold for log-likelihood ratio of foreground over background
        # if llr is >= thresh_low and near a pixel > thresh then don't 
        # include in the background model
        self.expbgfgmodel_llr_thresh = 0
        self.expbgfgmodel_llr_thresh_low = 0
        
        # fraction of sampled frames that we require to be classified as background according
        # to the prior expbgfgmodel to trust our estimates. otherwise, we will fill missing sections
        # either with the prior mean and standard deviation or by interpolation 
        self.min_frac_frames_isback = .1
        self.expbgfgmodel_fill = 'Interpolation'
        self.EXPBGFGMODEL_FILL_STRINGS = ['Prior BG Model','Interpolation']

        # number of foreground pixels to sample per location
        self.prior_fg_nsamples_pool = 25
        # number of background pixels to sample per location
        self.prior_bg_nsamples_pool = 25

        # how much to increase the sample radius per iteration
        self.prior_fg_pool_radius_factor = 1.25
        # how much to increase the sample radius per iteration
        self.prior_bg_pool_radius_factor = 1.25

        # whether to use our uncompressed AVI reader
        self.use_uncompressed_avi = True

    def enable_feedback( self, now_enabled ):
        """Change enablement of GUI/user feedback, dependent on interactivity mode.""" 
        self.feedback_enabled = now_enabled and self.interactive

    def __print__(self):
        s = ""
        for i,j in self.__dict__.iteritems():
            if j is None:
                s += i + ": None\n"
            else:
                s += i + ": " + str(j) + "\n"
        return s

    def __repr__( self ): return self.__print__()

    def __str__( self ): return self.__print__()

    def copy(self):
        v = Parameters()
        for i,j in self.__dict__.iteritems():
            if i == 'movie':
                continue
            try:
                v.__dict__[i] = copy.deepcopy(j)
            except:
                v.__dict__[i] = j
        return v

    def __eq__( self, other ):
        for i,j in self.__dict__.iteritems():
            if not hasattr(other,i):
                return False
            if not (j == other.__dict__[i]):
                return False
        for i,j in other.__dict__.iteritems():
            if not hasattr(self,i):
                return False
            if not (j == self.__dict__[i]):
                return False
        return True
        
params = Parameters()

diagnostics = dict()
diagnostics['nbirths_nohindsight'] = 0
diagnostics['ndeaths_nohindsight'] = 0
diagnostics['ndeaths_notfixed'] = 0
diagnostics['nbirths_notfixed'] = 0
diagnostics['nsplits_fixed'] = 0
diagnostics['nspurious_fixed'] = 0
diagnostics['nmerged_fixed'] = 0
diagnostics['nlost_fixed'] = 0
diagnostics['nhindsight_fixed'] = 0
diagnostics['nlarge_notfixed'] = 0
diagnostics['nsmall_notfixed'] = 0
diagnostics['nlarge_split'] = 0
diagnostics['nsmall_merged'] = 0
diagnostics['nsmall_lowerthresh'] = 0
diagnostics['nsmall_deleted'] = 0
diagnostics['max_nsplit'] = 0
diagnostics['sum_nsplit'] = 0
diagnostics['nlarge_ignored'] = 0
diagnostics['nframes_analyzed'] = 0

class GUIConstants:
    def __init__( self ):
        self.info = AboutDialogInfo()
        self.info.SetName( "Ctrax" )
        self.info.SetVersion( __version__ )
        self.info.SetCopyright( "2007-2013, Caltech ethomics project" )
        self.info.SetDescription( """The Caltech Multiple Fly Tracker
Kristin Branson et al.

http://ctrax.sourceforge.net/
http://dx.doi.org/10.1038/nmeth.1328

Distributed under the GNU General Public License
(http://www.gnu.org/licenses/gpl.html) with
ABSOLUTELY NO WARRANTY.

This project has been supported by the NIH and
the HHMI.""" )
        
        self.TRACK_START = "Start Tracking"
        self.TRACK_STOP = "Stop Tracking"
        self.TRACK_PLAY = "Start Playback"
        self.TRACK_PAUSE = "Stop Playback"

        params.version = __version__
# rather than global variables for each of these...

const = GUIConstants()



# fixbg.py
# region-of-interest modification dialog for background modeling
# JAB 7/30/11 (made from a previously pasted-and-modified version of roi.py)

import wx
import numpy as num
import scipy.ndimage.morphology as morph

import imagesk
import roi


class FixBG (roi.ROI):
    def __init__( self, parent, bgmodel ):
        roi.ROI.__init__( self, parent, bgmodel, init_polygons=bgmodel.fixbg_polygons )

        self.bgcenter = bgmodel.center.copy()
        self.bgdev = bgmodel.dev.copy()
        self.bgcenter0 = self.bgcenter.copy()
        self.bgdev0 = self.bgdev.copy()

        self.undo_data = bgmodel.fixbg_undo_data

        self.displaymode_choice.SetString( 1, "Background Deviation" )
        self.frame.SetTitle( "Fix Background Model" )


    def make_instruction_text( self ):
        """Get default (ROI) user instructions and modify them for bg-fixing."""
        lines = roi.ROI.make_instruction_text( self )
        for ll in range( len( lines ) ):
            line = lines[ll]
            if line[0:2] == '1.':
                line = '1. Click to select a polygonal region of background image to fix.'
            elif line[0:2] == '4.':
                line = '4. Background within polygon will be interpolated from boundaries.'
            elif line[0:2] == '5.':
                line = '5. Push "Save" button to save the current background model.'
            lines[ll] = line
        return lines


    def AddPolygon(self):
        wx.BeginBusyCursor()
        wx.Yield()
        self.currpolygon = num.r_[self.currpolygon,num.reshape(self.currpolygon[0,:],(1,2))]
        self.polygons.append(self.currpolygon)
        
        isin = self.fix_hole(self.bgcenter,self.bgcenter,self.currpolygon)
        undo_bg_coords = isin
        isin = self.fix_hole(self.bgdev,self.bgdev,self.currpolygon,isin=isin)
        undo_dev_coords = isin

        # save data for later undo
        undo_bg = self.bgcenter0[undo_bg_coords]
        undo_dev = self.bgdev0[undo_dev_coords]
        self.undo_data.append( {'bg_data': undo_bg, 'bg_coords': undo_bg_coords,
                                'dev_data': undo_dev, 'dev_coords': undo_dev_coords} )

        self.issaved = False
        wx.EndBusyCursor()

        
    def OnSave(self,evt=None):
        self.bgmodel.SetCenter(self.bgcenter)
        self.bgmodel.SetDev(self.bgdev)
        self.bgmodel.SetFixBgPolygons(self.polygons,self.undo_data)
        
        self.issaved = True


    def undo_last( self ):
        lastpolygon = self.polygons.pop()
        self.RemovePolygon(lastpolygon)


    def get_image_to_show( self ):
        if self.displaymode_choice.GetSelection() == 0:
            return imagesk.double2mono8(self.bgcenter,donormalize=True)
        else:
            return imagesk.double2mono8(self.bgdev,donormalize=True)


    def RemovePolygon(self,poly):
        wx.BeginBusyCursor()
        wx.Yield()
        #isin = self.undo_fix_hole(self.bgcenter0,self.bgcenter,poly)
        #isin = self.undo_fix_hole(self.bgdev0,self.bgdev,poly,isin=isin)
        undo_data = self.undo_data.pop()
        self.bgcenter[undo_data['bg_coords']] = undo_data['bg_data']
        self.bgdev[undo_data['dev_coords']] = undo_data['dev_data']
        self.issaved = False
        wx.EndBusyCursor()


    def fix_hole(self,im,out,polygon,isin=None):
        # get points on the inside of the polygon
        if isin is None:
            isin = roi.point_inside_polygon(self.X,self.Y,polygon)
        
        return fix_holes(im,out,isin)
    

    def undo_fix_hole(self,im0,im,polygon,isin=None):
        if isin is None:
            isin = roi.point_inside_polygon(self.X,self.Y,polygon)
        im[isin] = im0[isin]    
        return isin

        
def fix_holes(im,out,isin):

    if isin.all():
        raise ValueError('Cannot fix holes. All pixels are labeled as holes.')

    isout = (isin == False)

    # strel for eroding
    se = num.ones((3,3),bool)

    # store dilated version here
    isout1 = num.zeros(isout.shape,dtype=bool)

    # loop, dilating known regions
    while not isout.all():
        
        # get pixels just inside the border
        morph.binary_dilation(isout,se,output=isout1)
        border = isout1 & num.logical_not(isout)
        (yb,xb) = num.nonzero(border)
        yn = num.vstack([yb-1,
                         yb-1,
                         yb-1,
                         yb,
                         yb,
                         yb+1,
                         yb+1,
                         yb+1])
        xn = num.vstack([xb-1,
                         xb,
                         xb+1,
                         xb-1,
                         xb+1,
                         xb-1,
                         xb,
                         xb+1])
        badidx = num.logical_or(yn >= im.shape[0],
                                num.logical_or(yn < 0,
                                               num.logical_or(xn >= im.shape[1],
                                                              xn < 0)))
        yn = yn[badidx == False]
        xn = xn[badidx == False]
        #print "xn = " + str(xn)
        #print "yn = " + str(yn)
        #print "isout[yn,xn] = " + str(isout[yn,xn].astype(int))
        out[yb,xb] = num.average(out[yn,xn],axis=0,weights=isout[yn,xn].astype(float))

#        plt.subplot(121)
#        plt.imshow(isout)
#        plt.subplot(122)
#        plt.imshow(out)
#        plt.show()
        
        # swap isout, isout1x
        isout2 = isout1
        isout1 = isout
        isout = isout2

    return isin


def fix_hole(im,out,polygon,isin=None,X=None,Y=None):

    s = num.ones((3,3),bool)        

    # get points on the inside of the polygon
    if isin is None:
        if X is None or Y is None:
            (Y,X) = num.mgrid[0:im.shape[0],0:im.shape[1]]
        isin = roi.point_inside_polygon(X,Y,polygon)
        
    (y_isin,x_isin) = num.nonzero(isin)

    # get points on the outside boundary of the polygon
    isboundary = num.logical_and(morph.binary_dilation(isin,s),
                                 ~isin)

    (y_isboundary,x_isboundary) = num.nonzero(isboundary)

    # for each point inside the polygon, get an average from
    # all the boundary points
    for i in range(len(y_isin)):
        x = x_isin[i]
        y = y_isin[i]
        d = num.sqrt((y_isboundary-y)**2+(x_isboundary-x)**2)
        w = num.exp(-d)
        out[y,x] = num.sum(im[isboundary] * w) / num.sum(w)

    return isin




import codedir
import copy
import os
import sys

import motmot.wxvideo.wxvideo as wxvideo
import motmot.wxvalidatedtext.wxvalidatedtext as wxvt # part of Motmot
import numpy as num
#from scipy.misc.pilutil import imresize
import wx
from wx import xrc

import params
import ellipsesk as ell
from ellipses_draw import draw_ellipses
import imagesk
if 'darwin' in sys.platform:
    from mac_text_fixer import fix_text_sizes
from matchidentities import cvpred
from version import DEBUG_TRACKINGSETTINGS

RSRC_FILE = os.path.join(codedir.codedir,'xrc','tracking_settings.xrc')

SHOW_UNFILTERED_OBSERVATIONS = 0
SHOW_FILTERED_OBSERVATIONS = 1
SHOW_SMALL_OBSERVATIONS = 2
SHOW_LARGE_OBSERVATIONS = 3
SHOW_DELETED_OBSERVATIONS = 4
SHOW_SPLIT_OBSERVATIONS = 5
SHOW_MERGED_OBSERVATIONS = 6
SHOW_LOWERED_OBSERVATIONS = 7
SHOW_MAXJUMP = 8
SHOW_MOTIONMODEL = 9

class StoredObservations:

    def __init__(self,obs,frame,ellsmall=None,elllarge=None,didlowerthresh=None,
                 didmerge=None,diddelete=None,didsplit=None):

        self.obs = obs
        self.frame = frame
        #self.params = params.params.copy()
        #self.params = None
        self.isvalid = True
        self.ellsmall = ellsmall
        self.elllarge = elllarge
        self.didlowerthresh = didlowerthresh
        self.didmerge = didmerge
        self.diddelete = diddelete
        self.didsplit = didsplit


    def SetInvalid(self):
      self.isvalid = False

    def issame(self,frame):
        return (self.isvalid and (self.frame == frame))

    def __str__(self):
      s = ''
      if hasattr(self,'isvalid') and self.isvalid is not None:
        s += 'isvalid = ' + str(self.isvalid) + '\n'
      if hasattr(self,'frame') and self.frame is not None:
        s += 'frame = ' + str(self.frame) + '\n'
      if hasattr(self,'obs') and self.obs is not None:
        s += 'obs = ' + str(self.obs) + ', len = ' + str(len(self.obs)) + '\n'
      if hasattr(self,'ellsmall') and self.ellsmall is not None:
        s += 'ellsmall = ' + str(self.ellsmall) + '\n'
      if hasattr(self,'elllarge') and self.elllarge is not None:
        s += 'elllarge = ' + str(self.elllarge) + '\n'
      if hasattr(self,'didlowerthresh') and self.didlowerthresh is not None:
        s += 'didlowerthresh = ' + str(self.didlowerthresh) + '\n'
      if hasattr(self,'didmerge') and self.didmerge is not None:
        s += 'didmerge = ' + str(self.didmerge) + '\n'
      if hasattr(self,'diddelete') and self.diddelete is not None:
        s += 'diddelete = ' + str(self.diddelete) + '\n'
      if hasattr(self,'didsplit') and self.didsplit is not None:
        s += 'didsplit = ' + str(self.didsplit) + '\n'

      return s

class TrackingSettings:

    def __init__(self,parent,bg_imgs,currframe):

        self.parent = parent
        # background model
        self.bg_imgs = bg_imgs
        self.show_frame = currframe
        self.bg_img_frame = -1
        # zoom mode
        self.zoomin = False
        self.zoomout = False
        self.info = False
        self.zoomaxes = [0,params.params.movie_size[1]-1,0,params.params.movie_size[0]-1]
        self.zoomfactor = 1

        self.shape_uptodate = False
        self.automatic_minshape = params.ShapeParams()
        self.automatic_maxshape = params.ShapeParams()
        self.automatic_meanshape = params.ShapeParams()

        rsrc = xrc.XmlResource( RSRC_FILE )
        self.frame = rsrc.LoadFrame(parent,"trackingframe")

        if 'darwin' in sys.platform:
            fix_text_sizes( self.frame )

        self.InitControlHandles()
        self.InitializeValues()
        self.BindCallbacks()
        self.OnResize()
        self.ShowImage()

    def RegisterParamChange(self):
      if hasattr(self,'obs_filtered'):
        self.obs_filtered.SetInvalid()
      if hasattr(self,'obs_unfiltered'):
        self.obs_unfiltered.SetInvalid()
      if hasattr(self,'obs_prev'):
        self.obs_prev.SetInvalid()

    def InitControlHandles(self):

        #self.min_ntargets_input = self.control('min_ntarets')
        #self.max_ntargets_input = self.control('max_ntargets')
        self.automatic_shape_input = self.control('automatic_bounds')
        self.automatic_panel = self.control('automatic_panel')
        self.nstd_shape_input = self.control('nstd_shape')
        self.nframes_shape_input = self.control('nframes_shape')
        self.compute_shape_input = self.control('compute_shape')
        self.automatic_shape_text = self.control('automatic_shape_text')
        self.manual_shape_input = self.control('manual_bounds')
        self.manual_panel = self.control('manual_panel')
        self.min_area_input = self.control('min_area')
        self.mean_area_input = self.control('mean_area')
        self.max_area_input = self.control('max_area')
        self.min_major_input = self.control('min_major')
        self.mean_major_input = self.control('mean_major')
        self.max_major_input = self.control('max_major')
        self.min_minor_input = self.control('min_minor')
        self.mean_minor_input = self.control('mean_minor')
        self.max_minor_input = self.control('max_minor')
        self.min_ecc_input = self.control('min_ecc')
        self.mean_ecc_input = self.control('mean_ecc')
        self.max_ecc_input = self.control('max_ecc')
        self.angle_weight_input = self.control('angle_weight')
        self.max_jump_input = self.control('max_jump')
        # KB 20120109: allow jumps of distance max_jump_split if an observation is the result of splitting a connected component
        self.max_jump_split_input = self.control('max_jump_split')
        self.min_jump_input = self.control('min_jump')
        self.center_dampen_input = self.control('center_dampen')
        self.angle_dampen_input = self.control('angle_dampen')
        self.max_area_delete_input = self.control('max_area_delete')
        self.min_area_ignore_input = self.control('min_area_ignore')
        self.max_penalty_merge_input = self.control('max_penalty_merge')
        self.lower_thresh_input = self.control('lower_thresh')
        self.maxclustersperblob_input = self.control('maxclustersperblob')
        self.max_blobs_input = self.control( 'max_blobs_detect' )
        self.done_input = self.control('done')
        self.img_panel = self.control('img_panel')
        self.frame_scrollbar = self.control('frame_scrollbar')
        self.frame_number_text = self.control('frame_number')
        self.show_img_input = self.control('show_img')
        self.toolbar = xrc.XRCCTRL(self.frame,'toolbar')
        if self.toolbar is None:
            self.toolbar = self.frame.GetToolBar()
        self.zoomin_id = xrc.XRCID('zoomin')
        self.zoomout_id = xrc.XRCID('zoomout')
        self.info_id = xrc.XRCID('moreinfo')
        self.info_text = xrc.XRCCTRL( self.frame, "text_obsinfo" )
##        if 'win' in sys.platform:
##            self.toolbar.AddSeparator()
##            self.info_text = wx.TextCtrl(self.toolbar, -1, 'Observation Info', pos=(80,0),size=(300,20),style=wx.TE_READONLY|wx.TE_CENTRE)
##            self.toolbar.AddControl(self.info_text)
##        else:
##            self.info_text = wx.TextCtrl(self.toolbar, -1, 'Observation Info', size=(300,20),style=wx.TE_READONLY|wx.TE_CENTRE)
        self.info_text.SetValue('Observation Info')
        #self.info_text.SetEditable(False)
        self.hindsight_panel = self.control('hindsight_panel')
        self.splitdetection_panel = self.control('splitdetection_panel')
        self.mergeddetection_panel = self.control('mergeddetection_panel')
        self.spuriousdetection_panel = self.control('spuriousdetection_panel')
        self.lostdetection_panel = self.control('lostdetection_panel')
        self.do_fix_split_input = self.control('do_fix_split')
        self.do_fix_merged_input = self.control('do_fix_merged')
        self.do_fix_spurious_input = self.control('do_fix_spurious')
        self.do_fix_lost_input = self.control('do_fix_lost')
        self.splitdetection_length_input = self.control('splitdetection_length')
        self.splitdetection_cost_input = self.control('splitdetection_distance')
        self.mergeddetection_length_input = self.control('mergeddetection_length')
        self.mergeddetection_distance_input = self.control('mergeddetection_distance')
        self.spuriousdetection_length_input = self.control('spuriousdetection_length')
        self.lostdetection_length_input = self.control('lostdetection_length')

        box = wx.BoxSizer( wx.VERTICAL )
        self.img_panel.SetSizer( box )
        self.img_wind = wxvideo.DynamicImageCanvas( self.img_panel, -1 )
        self.img_wind.set_resize(True)
        box.Add( self.img_wind, 1, wx.EXPAND )
        self.img_panel.SetAutoLayout( True )
        self.img_panel.Layout()

    def InitializeValues(self):
        #self.min_ntargets_input.SetValue(str(params.params.min_ntargets))
        #self.max_ntargets_input.SetValue(str(params.params.max_ntargets))
        self.automatic_shape_input.SetValue(True)
        self.manual_shape_input.SetValue(False)
        self.nstd_shape_input.SetValue(str(params.params.n_std_thresh))
        self.nframes_shape_input.SetValue(str(params.params.n_frames_size))
        self.min_area_input.SetValue(str(params.params.minshape.area))
        self.min_major_input.SetValue(str(params.params.minshape.major))
        self.min_minor_input.SetValue(str(params.params.minshape.minor))
        self.min_ecc_input.SetValue(str(params.params.minshape.ecc))
        self.mean_area_input.SetValue(str(params.params.meanshape.area))
        self.mean_major_input.SetValue(str(params.params.meanshape.major))
        self.mean_minor_input.SetValue(str(params.params.meanshape.minor))
        self.mean_ecc_input.SetValue(str(params.params.meanshape.ecc))
        self.max_area_input.SetValue(str(params.params.maxshape.area))
        self.max_major_input.SetValue(str(params.params.maxshape.major))
        self.max_minor_input.SetValue(str(params.params.maxshape.minor))
        self.max_ecc_input.SetValue(str(params.params.maxshape.ecc))
        self.angle_dampen_input.SetValue(str(params.params.ang_dist_wt))
        self.max_area_delete_input.SetValue(str(params.params.maxareadelete))
        self.min_area_ignore_input.SetValue(str(params.params.minareaignore))
        self.max_penalty_merge_input.SetValue(str(params.params.maxpenaltymerge))
        self.max_jump_input.SetValue(str(params.params.max_jump))
        # KB 20120109: allow jumps of distance max_jump_split if an observation is the result of splitting a connected component
        self.max_jump_split_input.SetValue(str(params.params.max_jump_split))
        self.min_jump_input.SetValue(str(params.params.min_jump))
        self.angle_weight_input.SetValue(str(params.params.ang_dist_wt))
        self.center_dampen_input.SetValue(str(params.params.dampen))
        self.angle_dampen_input.SetValue(str(params.params.angle_dampen))
        self.lower_thresh_input.SetValue(str(params.params.minbackthresh))
        self.maxclustersperblob_input.SetValue(str(params.params.maxclustersperblob))
        self.max_blobs_input.SetValue( str( params.params.max_n_clusters ) )
        self.frame_scrollbar.SetThumbPosition(self.show_frame)
        self.frame_scrollbar.SetScrollbar(self.show_frame,0,params.params.n_frames-1,30)
        self.img_chosen = SHOW_UNFILTERED_OBSERVATIONS
        self.show_img_input.SetSelection(self.img_chosen)
        self.toolbar.SetToggle(self.zoomin,self.zoomin_id)
        self.toolbar.SetToggle(self.zoomout,self.zoomout_id)
        self.toolbar.SetToggle(self.info,self.info_id)

        self.do_fix_split_input.SetValue(params.params.do_fix_split)
        self.do_fix_merged_input.SetValue(params.params.do_fix_merged)
        self.do_fix_spurious_input.SetValue(params.params.do_fix_spurious)
        self.do_fix_lost_input.SetValue(params.params.do_fix_lost)
        self.splitdetection_panel.Enable(params.params.do_fix_split)
        self.mergeddetection_panel.Enable(params.params.do_fix_merged)
        self.spuriousdetection_panel.Enable(params.params.do_fix_spurious)
        self.lostdetection_panel.Enable(params.params.do_fix_lost)
        
        self.splitdetection_length_input.SetValue(str(params.params.splitdetection_length))
        self.splitdetection_cost_input.SetValue('%.2f'%params.params.splitdetection_cost)
        self.mergeddetection_length_input.SetValue(str(params.params.mergeddetection_length))
        self.mergeddetection_distance_input.SetValue('%.2f'%params.params.mergeddetection_distance)
        self.spuriousdetection_length_input.SetValue(str(params.params.spuriousdetection_length))
        self.lostdetection_length_input.SetValue(str(params.params.lostdetection_length))

        
    def BindCallbacks(self):

        # number of targets
        #self.bindctrl(('min_ntargets','max_ntargets'),('int','int'),self.SetNTargets)

        # shape
        
        # automatic computation of bounds on shape
        self.frame.Bind(wx.EVT_RADIOBUTTON,self.OnAutomatic,self.automatic_shape_input)
        self.frame.Bind(wx.EVT_RADIOBUTTON,self.OnAutomatic,self.manual_shape_input)

        # automatic shape
        self.bindctrl(('nstd_shape','nframes_shape'),('float','float'),self.SetAutomatic)

        # compute shape now
        self.frame.Bind(wx.EVT_BUTTON,self.ComputeShapeNow,self.compute_shape_input)

        # manual shape
        self.bindctrl(('min_area','mean_area','max_area',
                       'min_major','mean_major','max_major',
                       'min_minor','mean_minor','max_minor',
                       'min_ecc','mean_ecc','max_ecc'),
                      ('float','float','float',
                       'float','float','float',
                       'float','float','float',
                       'float','float','float'),
                      self.SetManual)

        # motion

		# KB 20120109: allow jumps of distance max_jump_split if an observation is the result of 
		# splitting a connected component
        self.bindctrl(('angle_weight','max_jump','max_jump_split','min_jump','center_dampen','angle_dampen'),
                      ('float','float','float','float','float','float'),
                      self.SetMotion)

        # observation

        self.bindctrl(('max_area_delete','min_area_ignore','max_penalty_merge','lower_thresh','maxclustersperblob', 'max_blobs_detect'),
                      ('float','float','float','float','int','int'),self.SetObservation)


        # hindsight
        self.frame.Bind(wx.EVT_CHECKBOX,self.OnSplit,self.do_fix_split_input)
        self.frame.Bind(wx.EVT_CHECKBOX,self.OnMerged,self.do_fix_merged_input)
        self.frame.Bind(wx.EVT_CHECKBOX,self.OnSpurious,self.do_fix_spurious_input)
        self.frame.Bind(wx.EVT_CHECKBOX,self.OnLost,self.do_fix_lost_input)
        self.bindctrl(('splitdetection_length','splitdetection_distance'),
                      ('int','float'),self.SetSplit)
        self.bindctrl(('mergeddetection_length','mergeddetection_distance'),
                      ('int','float'),self.SetMerged)
        self.bindctrl(('spuriousdetection_length',),('int',),self.SetSpurious)
        self.bindctrl(('lostdetection_length',),('int',),self.SetLost)
        
        # frame scrollbar
        self.frame.Bind(wx.EVT_SCROLL,self.FrameScrollbarMoved,self.frame_scrollbar)

        # img choice 
        self.frame.Bind(wx.EVT_CHOICE,self.ImageChosen,self.show_img_input)

        # resize
        self.frame.Bind( wx.EVT_SIZE, self.OnResize )
        #self.frame.Bind( wx.EVT_MOVE, self.OnResizeBG )

        # toolbar
        self.frame.Bind(wx.EVT_TOOL, self.ZoominToggle, id=self.zoomin_id)
        self.frame.Bind(wx.EVT_TOOL, self.ZoomoutToggle, id=self.zoomout_id)
        self.frame.Bind(wx.EVT_TOOL, self.InfoToggle, id=self.info_id)

        # mouse click
        # get a pointer to the "trackset" child
        self.img_size = params.params.movie_size
        img = num.zeros((self.img_size[0],self.img_size[1]),dtype=num.uint8)
        try:
            self.img_wind.update_image_and_drawings( "trackset", img, format="MONO8" )
        except:
            print "tracking_settings passing on redraw error, size:", self.img_wind.GetSize()
        self.img_wind_child = self.img_wind.get_child_canvas("trackset")
        self.img_wind_child.Bind(wx.EVT_LEFT_DOWN,self.MouseClick)
        self.img_wind_child.Bind(wx.EVT_LEFT_DCLICK,self.MouseDoubleClick)

    def control(self,ctrlname):
        return xrc.XRCCTRL(self.frame,ctrlname)

    def bindctrl(self,ctrlnames,type,validatef):
        for i,v in enumerate(ctrlnames):
            if type[i] == 'int':
                wxvt.setup_validated_integer_callback(self.control(v),
                                                      xrc.XRCID(v),
                                                      validatef,
                                                      pending_color=params.params.wxvt_bg)
            else:
                wxvt.setup_validated_float_callback(self.control(v),
                                                    xrc.XRCID(v),
                                                    validatef,
                                                    pending_color=params.params.wxvt_bg)

    def OnAutomatic(self,evt):

        isautomatic = self.automatic_shape_input.GetValue()
        if isautomatic == self.manual_shape_input.GetValue():
            print 'error: isautomatic == ismanual'
        if isautomatic:
            self.automatic_panel.Enable(True)
            self.manual_panel.Enable(False)
            if not self.shape_uptodate:
                self.automatic_shape_text.SetLabel('Bounds on shape not up to date.')
            else:
                self.automatic_shape_text.SetLabel('')
                params.params.minshape = self.automatic_minshape.copy()
                params.params.maxshape = self.automatic_maxshape.copy()
                params.params.meanshape = self.automatic_meanshape.copy()
                self.PrintShape()
                self.RegisterParamChange()
                self.ShowImage()
        else:
            self.automatic_panel.Enable(False)
            self.manual_panel.Enable(True)
            self.automatic_shape_text.SetLabel('')
            self.SetManual( evt ) # register current values

    def OnSplit(self,evt):

        params.params.do_fix_split = self.do_fix_split_input.GetValue()
        self.splitdetection_panel.Enable(params.params.do_fix_split)

    def OnMerged(self,evt):

        params.params.do_fix_merged = self.do_fix_merged_input.GetValue()
        self.mergeddetection_panel.Enable(params.params.do_fix_merged)

    def OnSpurious(self,evt):

        params.params.do_fix_spurious = self.do_fix_spurious_input.GetValue()
        self.spuriousdetection_panel.Enable(params.params.do_fix_spurious)

    def OnLost(self,evt):

        params.params.do_fix_lost = self.do_fix_lost_input.GetValue()
        self.lostdetection_panel.Enable(params.params.do_fix_lost)

    def SetSplit(self,evt):

        params.params.splitdetection_length = int(self.splitdetection_length_input.GetValue())
        params.params.splitdetection_cost = float(self.splitdetection_cost_input.GetValue())

    def SetMerged(self,evt):

        params.params.mergeddetection_length = int(self.mergeddetection_length_input.GetValue())
        params.params.mergeddetection_distance = float(self.mergeddetection_distance_input.GetValue())

    def SetSpurious(self,evt):

        params.params.spuriousdetection_length = int(self.spuriousdetection_length_input.GetValue())

    def SetLost(self,evt):

        params.params.lostdetection_length = int(self.lostdetection_length_input.GetValue())

    def SetAutomatic(self,evt):
        nstd = float(self.nstd_shape_input.GetValue())
        nframes = int(self.nframes_shape_input.GetValue())

        # check to make sure valid
        if nstd <= 0:
            self.nstd_shape_input.SetValue(str(params.params.n_std_thresh))
            nstd = params.params.n_std_thresh

        if nframes <= 0:
            self.nframes_shape_input.SetValue(str(params.params.n_frames_size))
            nframes = params.params.n_frames_size
        
        if (nstd != params.params.n_std_thresh) or (nframes != params.params.n_frames_size):
            params.params.n_std_thresh = nstd
            params.params.n_frames_size = nframes
            self.shape_uptodate = False
            self.automatic_shape_text.SetLabel('Bounds on shape not up to date.')

    def ComputeShapeNow(self,evt):

        # estimate shape now
        wx.BeginBusyCursor()
        wx.Yield()
        succeeded = ell.est_shape( self.bg_imgs,self.frame )
        wx.EndBusyCursor()
        
        if not succeeded:
          return

        # copy to temporary variable
        self.automatic_minshape = params.params.minshape.copy()
        self.automatic_maxshape = params.params.maxshape.copy()
        self.automatic_meanshape = params.params.meanshape.copy()

        # show shape
        self.PrintShape()

        # set up to date
        self.shape_uptodate = True
        self.automatic_shape_text.SetLabel('')
        self.RegisterParamChange()

        self.ShowImage()

    def PrintShape(self):

        self.min_area_input.SetValue(str(params.params.minshape.area))
        self.min_major_input.SetValue(str(params.params.minshape.major))
        self.min_minor_input.SetValue(str(params.params.minshape.minor))
        self.min_ecc_input.SetValue(str(params.params.minshape.ecc))
        self.mean_area_input.SetValue(str(params.params.meanshape.area))
        self.mean_major_input.SetValue(str(params.params.meanshape.major))
        self.mean_minor_input.SetValue(str(params.params.meanshape.minor))
        self.mean_ecc_input.SetValue(str(params.params.meanshape.ecc))
        self.max_area_input.SetValue(str(params.params.maxshape.area))
        self.max_major_input.SetValue(str(params.params.maxshape.major))
        self.max_minor_input.SetValue(str(params.params.maxshape.minor))
        self.max_ecc_input.SetValue(str(params.params.maxshape.ecc))

    def SetManual(self,evt):

        minarea = float(self.min_area_input.GetValue())
        meanarea = float(self.mean_area_input.GetValue())
        maxarea = float(self.max_area_input.GetValue())
        if (minarea <= meanarea) and (meanarea <= maxarea) and \
              (minarea >= 0):
            params.params.minshape.area = minarea
            params.params.meanshape.area = meanarea
            params.params.maxshape.area = maxarea
        else:
            self.min_area_input.SetValue(str(params.params.minshape.area))
            self.mean_area_input.SetValue(str(params.params.meanshape.area))
            self.max_area_input.SetValue(str(params.params.maxshape.area))
        minmajor = float(self.min_major_input.GetValue())
        meanmajor = float(self.mean_major_input.GetValue())
        maxmajor = float(self.max_major_input.GetValue())
        if (minmajor <= meanmajor) and (meanmajor <= maxmajor) and \
              (minmajor >= 0):
            params.params.minshape.major = minmajor
            params.params.meanshape.major = meanmajor
            params.params.maxshape.major = maxmajor
        else:
            self.min_major_input.SetValue(str(params.params.minshape.major))
            self.mean_major_input.SetValue(str(params.params.meanshape.major))
            self.max_major_input.SetValue(str(params.params.maxshape.major))
        minminor = float(self.min_minor_input.GetValue())
        meanminor = float(self.mean_minor_input.GetValue())
        maxminor = float(self.max_minor_input.GetValue())
        if (minminor <= meanminor) and (meanminor <= maxminor) and \
              (minminor >= 0):
            params.params.minshape.minor = minminor
            params.params.meanshape.minor = meanminor
            params.params.maxshape.minor = maxminor
        else:
            self.min_minor_input.SetValue(str(params.params.minshape.minor))
            self.mean_minor_input.SetValue(str(params.params.meanshape.minor))
            self.max_minor_input.SetValue(str(params.params.maxshape.minor))
        minecc = float(self.min_ecc_input.GetValue())
        meanecc = float(self.mean_ecc_input.GetValue())
        maxecc = float(self.max_ecc_input.GetValue())
        if (minecc <= meanecc) and (meanecc <= maxecc) and \
              (minecc >= 0) and (maxecc <= 1):
            params.params.minshape.ecc = minecc
            params.params.meanshape.ecc = meanecc
            params.params.maxshape.ecc = maxecc
        else:
            self.min_ecc_input.SetValue(str(params.params.minshape.ecc))
            self.mean_ecc_input.SetValue(str(params.params.meanshape.ecc))
            self.max_ecc_input.SetValue(str(params.params.maxshape.ecc))

        #params.params.meanshape = params.averageshape(params.params.minshape,
        #                                              params.params.maxshape)

        self.RegisterParamChange()
        self.ShowImage()

    def SetMotion(self,evt):
        
        angle_weight = float(self.angle_weight_input.GetValue())
        if angle_weight >= 0:
            params.params.ang_dist_wt = angle_weight
        else:
            self.angle_weight_input.SetValue(str(params.params.ang_dist_wt))

        max_jump = float(self.max_jump_input.GetValue())
        if max_jump > 0:
            params.params.max_jump = max_jump
        else:
            self.max_jump_input.SetValue(str(params.params.max_jump))

		# KB 20120109: allow jumps of distance max_jump_split if an observation is the result of 
		# splitting a connected component
        max_jump_split = float(self.max_jump_split_input.GetValue())
        if max_jump_split > 0:
            params.params.max_jump_split = max_jump_split
        else:
            self.max_jump_split_input.SetValue(str(params.params.max_jump_split))

        min_jump = float(self.min_jump_input.GetValue())
        if min_jump >= 0:
            params.params.min_jump = min_jump
        else:
            self.min_jump_input.SetValue(str(params.params.min_jump))

        center_dampen = float(self.center_dampen_input.GetValue())
        if center_dampen >= 0 and center_dampen <= 1:
            params.params.dampen = center_dampen
        else:
            self.center_dampen_input.SetValue(str(params.params.dampen))

        angle_dampen = float(self.angle_dampen_input.GetValue())
        if angle_dampen >= 0 and angle_dampen <= 1:
            params.params.angle_dampen = angle_dampen
        else:
            self.angle_dampen_input.SetValue(str(params.params.angle_dampen))

        self.RegisterParamChange()
        self.ShowImage()

    def SetObservation(self,evt):

        params.params.maxareadelete = float(self.max_area_delete_input.GetValue())
        params.params.minareaignore = float(self.min_area_ignore_input.GetValue())
        params.params.maxpenaltymerge = float(self.max_penalty_merge_input.GetValue())
        minbackthresh = float(self.lower_thresh_input.GetValue())
        maxclustersperblob = int(self.maxclustersperblob_input.GetValue())
        max_blobs_detect = int( self.max_blobs_input.GetValue() )
        if minbackthresh > 0 and minbackthresh <= 1:
            params.params.minbackthresh = minbackthresh
        else:
            self.lower_thresh_input.SetValue( str( params.params.minbackthresh ) )
        if maxclustersperblob >= 1:
            params.params.maxclustersperblob = maxclustersperblob
        else:
            self.maxclustersperblob_input.SetValue( str( params.params.maxclustersperblob ) )
        if max_blobs_detect >= 1:
            params.params.max_n_clusters = max_blobs_detect
        else:
            self.max_blobs_input.SetValue( str( params.params.max_n_clusters ) )
        self.RegisterParamChange()
        self.ShowImage()

    def FrameScrollbarMoved(self,evt):
	
	# get the value on the scrollbar now
	self.show_frame = self.frame_scrollbar.GetThumbPosition()

        if hasattr(self,'obs_filtered'):
            self.obs_prev = self.obs_filtered

	# show the image and update text
	self.ShowImage()


    def ShowImage(self):

        if DEBUG_TRACKINGSETTINGS: print 'ShowImage ' + str(self.show_frame)

        try:
            wx.Yield()
        except: pass # can be recursive sometimes in Windows
        wx.BeginBusyCursor()
        im, stamp = self.bg_imgs.movie.get_frame( int(self.show_frame) )

        windowsize = [self.img_panel.GetRect().GetHeight(),self.img_panel.GetRect().GetWidth()]

        self.GetBgImage()
        
        if self.img_chosen == SHOW_UNFILTERED_OBSERVATIONS:
            if DEBUG_TRACKINGSETTINGS: print 'SHOW_UNFILTERED_OBSERVATIONS'
            obs_unfiltered = self.GetObsUnfiltered()
            plot_linesegs = draw_ellipses(obs_unfiltered)
            
        elif self.img_chosen == SHOW_FILTERED_OBSERVATIONS:
            if DEBUG_TRACKINGSETTINGS: print 'SHOW_FILTERED_OBSERVATIONS'
            obs_filtered = self.GetObsFiltered()
            plot_linesegs = draw_ellipses(obs_filtered)
            
        elif self.img_chosen == SHOW_SMALL_OBSERVATIONS:
            if DEBUG_TRACKINGSETTINGS: print 'SHOW_SMALL_OBSERVATIONS'
            obs_unfiltered = self.GetObsUnfiltered()
            obs_small = []
            for obs in obs_unfiltered:
                if obs.area() < params.params.minshape.area:
                    obs_small.append(obs)
            plot_linesegs = draw_ellipses(obs_small)
            
        elif self.img_chosen == SHOW_LARGE_OBSERVATIONS:
            if DEBUG_TRACKINGSETTINGS: print 'SHOW_LARGE_OBSERVATIONS'
            obs_unfiltered = self.GetObsUnfiltered()
            obs_large = []
            for obs in obs_unfiltered:
                if obs.area() > params.params.maxshape.area:
                    obs_large.append(obs)
            plot_linesegs = draw_ellipses(obs_large)

        elif self.img_chosen == SHOW_DELETED_OBSERVATIONS:
            if DEBUG_TRACKINGSETTINGS: print 'SHOW_DELETED_OBSERVATIONS'
            (obs_unfiltered,obs_deleted) = self.GetObsUnfiltered('diddelete')
            plot_linesegs = draw_ellipses(obs_deleted)

        elif self.img_chosen == SHOW_SPLIT_OBSERVATIONS:
            if DEBUG_TRACKINGSETTINGS: print 'SHOW_SPLIT_OBSERVATIONS'
            (obs_unfiltered,obs_split) = self.GetObsUnfiltered('didsplit')
            plot_linesegs = draw_ellipses(obs_split)

        elif self.img_chosen == SHOW_MERGED_OBSERVATIONS:
            if DEBUG_TRACKINGSETTINGS: print 'SHOW_MERGED_OBSERVATIONS'
            (obs_unfiltered,obs_merge) = self.GetObsUnfiltered('didmerge')
            plot_linesegs = draw_ellipses(obs_merge)

        elif self.img_chosen == SHOW_LOWERED_OBSERVATIONS:
            if DEBUG_TRACKINGSETTINGS: print 'SHOW_LOWERED_OBSERVATIONS'
            (obs_unfiltered,obs_lowered) = self.GetObsUnfiltered('didlowerthresh')
            plot_linesegs = draw_ellipses(obs_lowered)

        # MAXJUMP
        elif self.img_chosen == SHOW_MAXJUMP:
            if DEBUG_TRACKINGSETTINGS: print 'SHOW_MAXJUMP'

            # either grab or compute observations
            obs_filtered = self.GetObsFiltered()
            plot_linesegs = draw_ellipses(obs_filtered)

            # draw circles
            for i,obs in enumerate(obs_filtered):
                plot_new_stuff = imagesk.draw_circle(obs.center.x,
                                                     obs.center.y,params.params.max_jump*2.,
                                                     params.params.colors[i%len(params.params.colors)])
                plot_linesegs.extend(plot_new_stuff)
                plot_new_stuff = imagesk.draw_circle(obs.center.x,
                                                     obs.center.y,params.params.min_jump,
                                                     params.params.colors[i%len(params.params.colors)])
                plot_linesegs.extend(plot_new_stuff)
                

        elif self.img_chosen == SHOW_MOTIONMODEL:
            if DEBUG_TRACKINGSETTINGS: print 'SHOW_MOTIONMODEL'

            (target_prev,target_curr,target_pred) = self.GetTargetMotion()

            # show the next frame, if there is one
            nextframe = num.minimum(int(self.show_frame+1),params.params.n_frames-1)
            im, stamp = self.bg_imgs.movie.get_frame(nextframe)

            # draw previous positions
            plot_linesegs = draw_ellipses(target_prev)

            # draw current positions
            plot_new_stuff = draw_ellipses(target_curr)
            plot_linesegs.extend(plot_new_stuff)

            # draw predicted positions
            plot_new_stuff = draw_ellipses(target_pred)
            plot_linesegs.extend(plot_new_stuff)

            scaleunit = params.params.max_jump / params.params.DRAW_MOTION_SCALE
            parcolor = [0,255,0]
            perpcolor = [0,255,0]
            anglecolor = [0,0,255]

            for i in target_pred.iterkeys():
                # compute direction of motion
                vx = target_pred[i].center.x - target_curr[i].center.x
                vy = target_pred[i].center.y - target_curr[i].center.y
                thetamotion = num.arctan2(vy,vx)

                # angle
                theta = target_pred[i].angle
                
                # we want to choose to add pi if that makes difference from motion direction smaller
                dtheta = abs( ((theta - thetamotion + num.pi) % (2.*num.pi)) - num.pi )
                if dtheta > (num.pi/2.):
                    theta += num.pi

                # compute end points of parallel motion
                x0 = target_pred[i].center.x + scaleunit*num.cos(theta)
                x1 = target_pred[i].center.x - scaleunit*num.cos(theta)
                y0 = target_pred[i].center.y + scaleunit*num.sin(theta)
                y1 = target_pred[i].center.y - scaleunit*num.sin(theta)

                # add parallel motion annotation line
                plot_new_stuff = imagesk.draw_line(x0+1,y0+1,x1+1,y1+1,parcolor)
                plot_linesegs.extend(plot_new_stuff)

                # compute end points of perpendicular motion
                x0 = target_pred[i].center.x + scaleunit*num.cos(num.pi/2.+theta)
                x1 = target_pred[i].center.x - scaleunit*num.cos(num.pi/2.+theta)
                y0 = target_pred[i].center.y + scaleunit*num.sin(num.pi/2.+theta)
                y1 = target_pred[i].center.y - scaleunit*num.sin(num.pi/2.+theta)
                
                # add perpendicular motion annotation line
                plot_new_stuff = imagesk.draw_line(x0+1,y0+1,x1+1,y1+1,perpcolor)
                plot_linesegs.extend(plot_new_stuff)
                

                # compute end points of angular motion
                if params.params.ang_dist_wt > 0:
                    
                    dtheta = scaleunit*num.sqrt(1./params.params.ang_dist_wt)
                    if dtheta >= (num.pi/2.):
                        print 'dtheta is more than pi/2'
                    else:
                        theta0 = theta - dtheta
                        theta1 = theta + dtheta
                        
                        # draw arc
                        plot_new_stuff = imagesk.draw_arc(target_pred[i].center.x,
                                                          target_pred[i].center.y,
                                                          scaleunit/2,
                                                          theta0,theta1,
                                                          anglecolor)
                        plot_linesegs.extend(plot_new_stuff)
                   
        im = imagesk.double2mono8(im,donormalize=False)
        linesegs,im = imagesk.zoom_linesegs_and_image(plot_linesegs,im,self.zoomaxes)
        (linesegs,linecolors) = imagesk.separate_linesegs_colors(linesegs)

        self.img_wind.update_image_and_drawings('trackset',
                                                im,
                                                format='MONO8',
                                                linesegs=linesegs,
                                                lineseg_colors=linecolors
                                                )
        self.img_wind.Refresh(eraseBackground=False)

        self.frame_number_text.SetLabel('Frame %d'%self.show_frame)
        
        if self.info == True:
            self.ShowInfo( None, None )

        wx.EndBusyCursor()

        if DEBUG_TRACKINGSETTINGS: sys.stdout.flush()
        

    def ImageChosen(self,evt):

        self.img_chosen = self.show_img_input.GetSelection()
        self.ShowImage()

    def OnResize(self,evt=None):

        if evt is not None: evt.Skip()
        self.frame.Layout()
        try:
            #self.ShowImage()
            self.frame_scrollbar.SetMinSize( wx.Size(
                self.img_panel.GetRect().GetWidth(),
                self.frame_scrollbar.GetRect().GetHeight() ) )
        except AttributeError: pass # during initialization

    def ZoominToggle(self,evt):

        self.zoomin = not self.zoomin
        if self.zoomin == True and self.zoomout == True:
            self.toolbar.ToggleTool(self.zoomout_id,False)
            self.zoomout = False
        if self.zoomin == True and self.info == True:
            self.toolbar.ToggleTool(self.info_id,False)
            self.info = False

    def ZoomoutToggle(self,evt):

        self.zoomout = not self.zoomout
        if self.zoomin == True and self.zoomout == True:
            self.toolbar.ToggleTool(self.zoomin_id,False)
            self.zoomin = False
        if self.zoomout == True and self.info == True:
            self.toolbar.ToggleTool(self.info_id,False)
            self.info = False

    def InfoToggle(self,evt):

        self.info = not self.info
        if self.info == True and self.zoomin == True:
            self.toolbar.ToggleTool(self.zoomin_id,False)
            self.zoomin = False
        if self.zoomout == True and self.info == True:
            self.toolbar.ToggleTool(self.zoomout_id,False)
            self.zoomout = False

    def MouseDoubleClick(self,evt):

        if self.zoomout == True:
            self.zoomfactor = 1
            self.SetZoomAxes()

    def MouseClick(self,evt):

        # get the clicked position
        resize = self.img_wind.get_resize()
        x = evt.GetX() / resize
        h = self.zoomaxes[3] - self.zoomaxes[2] + 1
        y = h - evt.GetY() / resize
          
        x += self.zoomaxes[0]
        y += self.zoomaxes[2]

        if (x > self.img_size[1]) or (y > self.img_size[0]):
            return
        if self.zoomin:
            self.ZoomIn(x,y)
        elif self.zoomout:
            self.ZoomOut()
        elif self.info:
            self.ShowInfo(x,y)
        
    def ZoomIn(self,x,y):

        self.zoomfactor *= 1.5
        self.zoompoint = [x,y]
        self.SetZoomAxes()

    def SetZoomAxes(self):

        x = self.zoompoint[0]
        y = self.zoompoint[1]
        
        W = params.params.movie_size[1]
        H = params.params.movie_size[0]
        h = H/self.zoomfactor
        w = W/self.zoomfactor
        x1 = x-w/2
        x2 = x+w/2
        y1 = y-h/2
        y2 = y+h/2

        if x1 < 0:
            x2 -= x1
            x1 = 0
        elif x2 > W-1:
            x1 -= (x2 - W + 1)
            x2 = W-1
        if y1 < 0:
            y2 -= y1
            y1 = 0
        elif y2 > H-1:
            y1 -= (y2 - H + 1)
            y2 = H-1
        x1 = num.maximum(int(x1),0)
        x2 = num.minimum(int(x2),W-1)
        y1 = num.maximum(int(y1),0)
        y2 = num.minimum(int(y2),H-1)

        self.zoomaxes = [x1,x2,y1,y2]
        self.ShowImage()

    def ZoomOut(self):

        if self.zoomfactor <= 1:
            return

        self.zoomfactor /= 1.5
        self.SetZoomAxes()

    def ShowInfo(self,x,y):

        # grab targets
        if (self.img_chosen == SHOW_MAXJUMP) or (self.img_chosen == SHOW_FILTERED_OBSERVATIONS):
            obs = self.obs_filtered.obs
        else:
            obs = self.obs_unfiltered.obs

        if (x is None or y is None):
            if hasattr( self, 'info_mini' ) and self.info_mini < len( obs ):
                mini = self.info_mini
                x = obs[mini].center.x
                y = obs[mini].center.y
            else:
                self.ShowObsInfo( None )
                return

        # determine closest target
        mind = num.inf
        for i,v in enumerate(obs):
            d = (v.center.x-x)**2 + (v.center.y-y)**2
            if d <= mind:
                mini = i
                mind = d

        maxdshowinfo = (num.maximum(self.zoomaxes[1]-self.zoomaxes[0],
                                    self.zoomaxes[2]-self.zoomaxes[1])/params.params.MAXDSHOWINFO)**2

        if mind < maxdshowinfo:
            self.ShowObsInfo(obs[mini])
            self.info_mini = mini
        else:
            self.ShowObsInfo( None )

        
    def ShowObsInfo(self,ellipse):
        """Output text describing currently selected ellipse."""
        if ellipse is None:
            self.info_text.SetValue( "" )
        else:
            self.info_text.SetValue('area=%.2f, maj=%.2f, min=%.2f, ecc=%.2f'%(ellipse.area(),ellipse.major,ellipse.minor,ellipse.eccentricity()))


    def GetObsFiltered(self):
        if not(hasattr(self,'obs_filtered') and self.obs_filtered.issame(self.show_frame)):
            if DEBUG_TRACKINGSETTINGS: print 'computing filtered observations'
            obs_filtered = ell.find_ellipses(self.bg_imgs.dfore.copy(),self.bg_imgs.cc.copy(),self.bg_imgs.ncc,True)
            self.obs_filtered = StoredObservations(obs_filtered,self.show_frame)
        else:
            if DEBUG_TRACKINGSETTINGS: print 'filtered observations already computed'
        if DEBUG_TRACKINGSETTINGS: print 'obs_filtered:\n' + str(self.obs_filtered)

        return self.obs_filtered.obs

    
    def GetObsUnfiltered(self,*args):

        # do we need to recompute?
        mustcompute = False
        if hasattr(self,'obs_unfiltered') and self.obs_unfiltered.issame(self.show_frame):
            for arg in args:
               if self.obs_unfiltered.__dict__[arg] is None:
                   mustcompute = True
                   break
        else:
            mustcompute = True

        if DEBUG_TRACKINGSETTINGS: print 'mustcompute = ' + str(mustcompute)
        if DEBUG_TRACKINGSETTINGS and not mustcompute:
          print 'stored obs_unfiltered = ' + str(self.obs_unfiltered)

        # if we are only interested in the unfiltered observation
        if len(args) == 0:
            # if it has not yet been computed for this frame, compute
            if mustcompute:
                obs_unfiltered = ell.find_ellipses(self.bg_imgs.dfore.copy(),self.bg_imgs.cc.copy(),self.bg_imgs.ncc,False)
                self.obs_unfiltered = StoredObservations(obs_unfiltered,self.show_frame)
            return self.obs_unfiltered.obs
          
        # compute if necessary
        if mustcompute:
            wx.BeginBusyCursor()
            wx.YieldIfNeeded()
            print "findellipsesdisplay"
            (obs_unfiltered,
             ellsmall,
             elllarge,
             didlowerthresh,
             didmerge,
             diddelete,
             didsplit) = ell.find_ellipses(self.bg_imgs.dfore.copy(),
                                           self.bg_imgs.cc.copy(),
                                           self.bg_imgs.ncc,
                                           return_vals=True)
            if DEBUG_TRACKINGSETTINGS: print 'computed obs_unfiltered = ' + str(obs_unfiltered) + ', len = ' + str(len(obs_unfiltered))
            if DEBUG_TRACKINGSETTINGS: print 'ellsmall = ' + str(ellsmall)
            if DEBUG_TRACKINGSETTINGS: print 'elllarge = ' + str(elllarge)
            if DEBUG_TRACKINGSETTINGS: print 'didlowerthresh = ' + str(didlowerthresh)
            if DEBUG_TRACKINGSETTINGS: print 'didmerge = ' + str(didmerge)
            if DEBUG_TRACKINGSETTINGS: print 'diddelete = ' + str(diddelete)
            if DEBUG_TRACKINGSETTINGS: print 'didsplit = ' + str(didsplit)
            wx.EndBusyCursor()
            self.obs_unfiltered = StoredObservations(obs_unfiltered,self.show_frame,
                                                     ellsmall,elllarge,didlowerthresh,
                                                     didmerge,diddelete,didsplit)
            if DEBUG_TRACKINGSETTINGS: print 'stored obs_unfiltered: '
            if DEBUG_TRACKINGSETTINGS: print str(self.obs_unfiltered)

        # create return list
        ret = (self.obs_unfiltered.obs,)
        for arg in args:
            ret += (self.obs_unfiltered.__dict__[arg],)

        return ret

    def GetBgImage(self):        
        if not (self.bg_img_frame == self.show_frame):
            (self.bg_imgs.dfore,self.bg_imgs.bw,
             self.bg_imgs.cc,self.bg_imgs.ncc) = \
             self.bg_imgs.sub_bg(self.show_frame)
            self.bg_img_frame = self.show_frame
        
    def GetObsPrev(self):
        if not(hasattr(self,'obs_prev') and self.obs_filtered.issame(self.show_frame-1)):
            wx.BeginBusyCursor()
            wx.Yield()
            prevframe = num.maximum(0,self.show_frame-1)
            (dfore,bw,cc,ncc) = self.bg_imgs.sub_bg(prevframe)
            obs_filtered = ell.find_ellipses(dfore,cc,ncc,True)
            wx.EndBusyCursor()
            self.obs_prev = StoredObservations(obs_filtered,self.show_frame)
        return self.obs_prev.obs

    def GetTargetMotion(self):

        # get current positions
        obs_curr = self.GetObsFiltered()
        # get previous positions
        obs_prev = self.GetObsPrev()
        # give identities to previous positions
        target_prev = ell.TargetList()
        for i,obs in enumerate(obs_prev):
            obs.identity = i
            target_prev.append(obs)
        # match previous and current targets, no velocity
        oldnids = params.params.nids
        target_curr = ell.find_flies(target_prev,target_prev,obs_curr)
        # don't actually assign new identities
        params.params.nids = oldnids
        # delete targets that aren't in both frames
        keyscurr = set(target_curr.keys())
        keysprev = set(target_prev.keys())
        keysremove = keyscurr - keysprev
        for i in keysremove:
            tmp = target_curr.pop(i)
        keysremove = keysprev - keyscurr
        for i in keysremove:
            tmp = target_prev.pop(i)
        
        # compute predicted positions
        target_pred = cvpred(target_prev,target_curr)
        # store
        targetmotion = (target_prev,target_curr,target_pred)
        # return
        return targetmotion



from params import params
params.interactive = False
import ExpBGFGModel
import re
import os
import os.path
import ParseExpDirs
import sys
import matplotlib.pyplot as plt

## parameters

mode = 'show'

# which experiments
protocol = '20110804'
expdirs_in = '/groups/branson/bransonlab/projects/olympiad/FlyBowlCtrax/20110804/LearnCtraxParams/expdirs.txt'
mindatestr = ''
maxdatestr = ''
linename = '.*'
rig = ''
plate = ''
bowl = ''
notstarted = False

print "*** PROTOCOL %s ***\n"%protocol

# directory for learning data
resdir = '/groups/branson/bransonlab/projects/olympiad/FlyBowlCtrax/%s/LearnCtraxParams'%protocol

# directory containing parameters
paramsdir = '/groups/branson/bransonlab/projects/olympiad/FlyBowlCtrax/%s'%protocol

# files within the learning data directory
paramsFileStr = 'ExpBGFGModelParams.txt'
movieFileStr = 'movie.ufmf'
annFileStr = 'movie.ufmf.ann'
expdirsFileStr = 'expdirs.txt'
outputFileStr = 'ExpBGFGModelResults.pickle'
matFileStr = 'ExpBGFGModelResults.mat'


# file to write experiment directory names to
expdirsFileName = os.path.join(resdir,expdirsFileStr)

# file containing parameters
paramsFileName = os.path.join(paramsdir,paramsFileStr)

# file to write results to
outputFileName = os.path.join(resdir,outputFileStr)

# mat file to write results to
matFileName = os.path.join(resdir,matFileStr)

# experiment directories corresponding to parameters
if expdirs_in == "":
    (expdirs,expdir_reads,expdir_writes,experiments) = \
        ParseExpDirs.getExpDirs(protocol=protocol,
                                mindatestr=mindatestr,
                                maxdatestr=maxdatestr,
                                linename=linename,
                                rig=rig,
                                plate=plate,
                                bowl=bowl,
                                notstarted=notstarted,
                                subreadfiles=[movieFileStr,annFileStr])
else:
    fid = open(expdirs_in,"r")
    expdirs = []
    experiments = []
    for expdir in fid:
        expdir = expdir.strip()
        if expdir == "":
            continue
        expdirs.append(expdir)
        [exp,success] = ParseExpDirs.parseExpDir(expdir)
        experiments.append(exp)
    expdir_reads = expdirs
    expdir_writes = expdirs
    fid.close()

if mode == 'learn':

    print "expdirs = " + str(expdirs)

    fid = open(expdirsFileName,"w")

    for expdir in expdir_reads:
        fid.write('%s\n'%expdir)

    fid.close()

    print 'execute the following command:'
    print 'python ExpBGFGModel.py' + \
        ' -f ' + expdirsFileName + \
        ' -p ' + paramsFileName + \
        ' -m ' + movieFileStr + \
        ' -a ' + annFileStr + \
        ' -o ' + outputFileName + \
        ' --mat ' + matFileName
    # we could just execute this command ourselves with os.system()...

elif mode == 'show':
    
    model = ExpBGFGModel.ExpBGFGModel(picklefile=outputFileName)
    model.show()
    for i in range(len(expdirs)):
        moviename = os.path.join(expdir_reads[i],movieFileStr)
        print moviename
        model.showtest(moviename=moviename)
        savename = os.path.join(resdir,'ExpBGFGModel_SampleFrames_%s.png'%os.path.basename(expdirs[i]))
        plt.savefig(savename,format='png')

elif mode == 'change':
    model = ExpBGFGModel.ExpBGFGModel(picklefile=outputFileName)
